#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


# fs_delete_success is part of fs_create_success() in this same directory

def access_group_remove_fail():
    errors = []

    init_id = read_var("LSM_INIT_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail removing initiator from AG without paramethers with protocol %s" % config[
                'protocol'], 'command': lsm.access_group_remove},
            {'message': "Trying to fail removing initiator from AG without ag with protocol %s" % config['protocol'],
             'command': lsm.access_group_remove, 'init': init_id},
            {'message': "Trying to fail removing initiator from AG without init with protocol %s" % config['protocol'],
             'command': lsm.access_group_remove, 'ag': ag_id},
            {'message': "Trying to fail removing initiator from AG with WRONG AG with protocol %s" % config['protocol'],
             'command': lsm.access_group_remove, 'ag': "WRONG", 'init': init_id},
            {'message': "Trying to fail removing initiator from AG with WRONG init with protocol %s" % config[
                'protocol'],
             'command': lsm.access_group_remove, 'ag': ag_id, 'init': "WRONG"}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def access_group_remove_fail_no_state_change():
    errors = []

    init_id_2 = read_var("LSM_INIT_ID_2")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Trying to fail removing wrong initiator from AG to hit NO_STATE_CHANGE "
                   "with protocol %s" % config['protocol'],
                   command=lsm.access_group_remove,
                   ag=ag_id,
                   init=init_id_2,
                   expected_ret=4,
                   errors=errors)
    return errors


def access_group_remove_last_init():
    errors = []

    init_id = read_var("LSM_INIT_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Trying to fail removing last initiator from AG to hit LAST_INIT_IN_ACCESS_GROUP "
                   "with protocol %s" % config['protocol'],
                   command=lsm.access_group_remove,
                   ag=ag_id,
                   init=init_id,
                   expected_ret=4,
                   errors=errors)

    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 2:
        errs = access_group_remove_fail()
        errs += access_group_remove_fail_no_state_change()
        errs += access_group_remove_last_init()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import libsan.host.iscsi as iscsi
from os import environ
from libsan.host.lio import TargetCLI
from stqe.host.persistent_vars import read_var, read_env
from stqe.host.atomic_run import atomic_run, parse_ret


def clean_target_chap_settigs():
    """
    Disabling CHAP on a target by removing CHAP parameters from groups auth and discovery_auth.
    :return:
    List of errors.
    """
    errors = []

    target = TargetCLI()
    tpg = read_var("TPG")
    target_iqn = read_var("TARGET_IQN")

    args = [{"userid": ''}, {"password": ''}]
    groups = ["auth", "discovery_auth"]

    for group in groups:
        if group == "auth":
            target.path = "/iscsi/" + target_iqn + "/" + tpg
        elif group == "discovery_auth":
            target.path = "/iscsi"
        for arg in args:
            atomic_run("Setting parameter: %s to '' in group %s" % (list(arg.keys())[0], group),
                       group=group,
                       command=target.set,
                       errors=errors,
                       **arg
                       )

    return errors


def iscsi_chap_pass():
    errors = []

    ip_address = environ["fmf_target_ip"]
    expected_ret = read_env("fmf_expected_ret")
    target_iqn = read_var("TARGET_IQN")
    username = str(read_var("CHAP_USERID"))
    password = str(read_var("CHAP_PASSWORD"))

    atomic_run("Setting CHAP username and password in iscsid config",
               target_user=username,
               target_pass=password,
               command=iscsi.set_chap,
               errors=errors
               )

    atomic_run("Discovering local target",
               target=ip_address,
               disc_db=True,
               command=iscsi.discovery_st,
               errors=errors
               )

    atomic_run("Trying to login to target: %s" % ip_address,
               expected_ret=expected_ret,
               target=target_iqn,
               command=iscsi.node_login,
               errors=errors
               )

    atomic_run("Cleaning up",
               command=iscsi.clean_up,
               errors=errors
               )

    atomic_run("Cleaning chap username and password iscsid config",
               command=iscsi.disable_chap,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = iscsi_chap_pass()
    errs += clean_target_chap_settigs()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def capabilities_success():
    errors = []

    sys = read_var("LSM_SYS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run("Getting capabilities of system %s with protocol %s" % (sys, config['protocol']),
                   command=lsm.capabilities,
                   sys=sys,
                   errors=errors)
    return errors


def capabilities_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail getting capabilities of system without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.capabilities},
            {'message': "Trying to fail getting capabilities of system with WRONG system with protocol %s" %
                        config['protocol'], 'sys': "WRONG", 'command': lsm.capabilities}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = capabilities_success()
    if int(environ['fmf_tier']) == 2:
        errs = capabilities_fail()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def _pool_test():
    global TestObj, vg_name

    # Test on 2 pools
    for pool_num in range(1, 3):
        # testing change discards with active pool and active thin volume
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "passdown"})

        # Change passdown <-> ignore should not work
        TestObj.tnok("lvchange --discards ignore %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "passdown"})

        # Change passdown <-> nopassdown is supported
        TestObj.tok("lvchange --discards nopassdown %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "nopassdown"})

        # Change nopassdown <-> ignore should not work
        TestObj.tnok("lvchange --discards ignore %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "nopassdown"})

        TestObj.tok("lvchange --discards passdown %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "passdown"})

        # For RHEL, since RHEL-6.7 the LVM attribute when is active should be twi-aotz--, before was twi-a-tz--
        expected_online_attr = "twi-aotz--"

        # testing change discards with inactive pool and inactive thin volume
        # Change passdown <-> ignore is supported
        TestObj.tok("lvchange -an %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -an %s/lv%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name,
                                {"discards": "passdown", "lv_attr": "twi---tz--"})
        TestObj.tok("lvchange --discards ignore %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/lv%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name,
                                {"discards": "ignore", "lv_attr": expected_online_attr})

        # Change ignore <-> nopassdown is supported
        TestObj.tok("lvchange -an %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -an %s/lv%d" % (vg_name, pool_num))
        TestObj.tok("lvchange --discards nopassdown %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/lv%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "nopassdown"})

        TestObj.tok("lvchange -an %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -an %s/lv%d" % (vg_name, pool_num))
        TestObj.tok("lvchange --discards ignore %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/lv%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "ignore"})

        # Change passdown <-> ignore is supported
        TestObj.tok("lvchange -an %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -an %s/lv%d" % (vg_name, pool_num))
        TestObj.tok("lvchange --discards passdown %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange -ay %s/lv%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"discards": "passdown"})

        # It is not possible to change thin pool to readonly
        err_msg = "Command not permitted on LV"
        if linux.dist_name() == "RHEL" and linux.dist_release() < "7.4":
            err_msg = "not yet supported"
        TestObj.tok("lvchange -p r %s/pool%d 2>&1 | grep '%s'" % (vg_name, pool_num, err_msg))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_attr": expected_online_attr})

        # refresh
        TestObj.tok("lvchange --refresh %s/pool%d" % (vg_name, pool_num))

        # monitor
        TestObj.tok("lvchange --monitor n %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvchange --monitor y %s/pool%d" % (vg_name, pool_num))

        # allocation policy
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_allocation_policy": "inherit"})
        TestObj.tok("lvchange -Cy %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_allocation_policy": "contiguous"})
        TestObj.tok("lvchange -Cn %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_allocation_policy": "inherit"})

        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_read_ahead": "auto"})

        # set read ahead to use 256 sectors
        TestObj.tok("lvchange -r 256 %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_read_ahead": "128.00k"})
        TestObj.tok("lvchange -r none %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_read_ahead": "0 "})
        TestObj.tok("lvchange -r auto %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_read_ahead": "auto"})

        # if zeroing is enabled
        TestObj.tok("lvchange -Zn %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"zero": ""})
        TestObj.tok("lvchange -Z y %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"zero": "zero"})


def _lv_test():
    # Test the 2 LVs

    for lv_num in range(1, 3):
        TestObj.tok("lvchange -an %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_attr": "Vwi---tz--"})
        TestObj.tnok("ls /dev/%s/lv%d" % (vg_name, lv_num))
        TestObj.tok("lvchange -a y %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_attr": "Vwi-a-tz--"})
        TestObj.tok("ls /dev/%s/lv%d" % (vg_name, lv_num))

        # change to readonly and back to rw
        TestObj.tok("lvchange -pr %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_attr": "Vri-a-tz--"})
        TestObj.tok("lvchange -p rw %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_attr": "Vwi-a-tz--"})

        TestObj.tok("lvchange --refresh %s/lv%d" % (vg_name, lv_num))

        # allocation policy
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_allocation_policy": "inherit"})
        TestObj.tok("lvchange -Cy %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_allocation_policy": "contiguous"})
        TestObj.tok("lvchange -Cn %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_allocation_policy": "inherit"})

        # set read ahead to use 256 sectors
        TestObj.tok("lvchange -r 256 %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_read_ahead": "128.00k"})
        TestObj.tok("lvchange -r none %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_read_ahead": "0 "})
        TestObj.tok("lvchange -r auto %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_read_ahead": "auto"})


def _snapshot_test():
    TestObj.tok("lvcreate -s %s/lv1 -n lv3" % vg_name)
    # snapshot is not active when created
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_attr": "Vwi---tz-k"})
    TestObj.tnok("ls /dev/%s/lv3" % vg_name)
    # must use -K to active snapshot
    TestObj.tok("lvchange -ay -K %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_attr": "Vwi-a-tz-k"})
    TestObj.tok("ls /dev/%s/lv3" % vg_name)

    TestObj.tok("lvchange -pr %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_attr": "Vri-a-tz-k"})
    TestObj.tok("lvchange -p rw %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_attr": "Vwi-a-tz-k"})

    TestObj.tok("lvchange --refresh %s/lv3" % vg_name)

    # allocation policy
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_allocation_policy": "inherit"})
    TestObj.tok("lvchange -Cy %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_allocation_policy": "contiguous"})
    TestObj.tok("lvchange -Cn %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_allocation_policy": "inherit"})

    # set read ahead to use 256 sectors
    TestObj.tok("lvchange -r 256 %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_read_ahead": "128.00k"})
    TestObj.tok("lvchange -r none %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_read_ahead": "0 "})
    TestObj.tok("lvchange -r auto %s/lv3" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"lv_read_ahead": "auto"})


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting LV Change Thin Provisioning test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    if TestObj.trun("lvcreate -V100m -L100m -T %s/pool1 -n lv1" % vg_name) != 0:
        TestObj.tfail("Could not Create pool1")
        return False

    if TestObj.trun("lvcreate -V100m -i2 -L100m -T %s/pool2 -n lv2" % vg_name) != 0:
        TestObj.tfail("Could not Create pool2")
        return False

    _pool_test()
    _lv_test()
    _snapshot_test()

    return True


def _clean_up():
    # TestObj.trun("lvremove -ff %s" % vg_name)
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        e = sys.exc_info()[0]
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_dependants_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    fs_cloned_id = read_var("LSM_FS_CLONED_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        # this should return 'True', because this FS has dependant - cloned FS
        ret, data = atomic_run("Checking dependants of FS %s with protocol %s." % (fs_id, config['protocol']),
                               command=lsm.fs_dependants,
                               fs=fs_id,
                               return_output=True,
                               errors=errors)
        if data != "True":
            errors.append("Checking dependants on FS %s did not return data 'True', but %s" % (fs_id, data))

        # this should return 'False', because this is the cloned FS without dependants
        ret, data = atomic_run("Checking dependants of FS %s with protocol %s." % (fs_cloned_id, config['protocol']),
                               command=lsm.fs_dependants,
                               fs=fs_cloned_id,
                               return_output=True,
                               errors=errors)
        if data != "False":
            errors.append("Checking dependants on FS %s did not return data 'False', but %s" % (fs_cloned_id, data))

        # FIXME: Add checking dependants for file
        # FIXME: Add tests with snapshots as dependants
    return errors


def fs_dependants_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail checking dependants without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.fs_dependants},
            {'message': "Trying to fail checking dependants FS with wrong 'fs' 'wrong' with protocol %s" % config['protocol'],
             'command': lsm.fs_dependants, 'fs': "wrong"},
            {'message': "Trying to fail checking dependants FS with 'fs' name instead of ID with protocol %s" % config['protocol'],
             'command': lsm.fs_dependants, 'fs': fs_name},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_dependants_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_dependants_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.persistent_vars import write_var
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.loopdev import create_loopdev
from os import environ


def remove_nones(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


def loopdev_create():
    errors = []

    args = dict(name=None, size=None)
    for arg in args.keys():
        try:
            args[arg] = environ['fmf_loopdev_%s' % arg]
        except KeyError:
            pass

    ret = atomic_run("Creating loopdev",
                     command=create_loopdev,
                     errors=errors,
                     **remove_nones(args)
                     )

    atomic_run("Writing var TARGETD_LOOPDEV_NAME",
               command=write_var,
               var={"TARGETD_LOOPDEV_NAME": ret},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = loopdev_create()
    exit(parse_ret(errs))



#!/usr/bin/python

#
# Related BZ:
#       1255552
#

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import stqe.host.tc
from stqe.host.atomic_run import atomic_run
from libsan.host.cmdline import run
from libsan.host.dmpd import thin_restore, thin_dump, thin_check, cache_dump, cache_check, cache_restore
from libsan.host.linux import install_package
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import vg_create, vg_remove, lv_create, lv_activate, lv_deactivate, lv_convert

TC = None


def _mount_thin_lv(vg_name, lv_name):
    run("mkdir -p /mnt/%s" % lv_name, verbose=True)
    run("mkfs.ext4 /dev/%s/%s" % (vg_name, lv_name), verbose=True)
    run("mount /dev/%s/%s /mnt/%s/" % (vg_name, lv_name, lv_name), verbose=True)


def _unmount_thin_lv(lv_name):
    run("umount -f /mnt/%s/" % lv_name, verbose=True)
    run("rm -rf /mnt/%s" % lv_name, verbose=True)


def create_lvs(args, iteration, errors):
    if iteration == 1:
        atomic_run("Creating thin LV.",
                   vg_name=args["group"] + "/" + args["pool"],
                   lv_name=args["vol"],
                   options=["-T", "-V 10"],
                   command=lv_create,
                   errors=errors)

        atomic_run("Deactivating thin LV.",
                   lv_name=args["vol"],
                   vg_name=args["group"],
                   command=lv_deactivate,
                   errors=errors)

    elif iteration == 2:
        atomic_run("Creating thin LV.",
                   vg_name=args["group"] + "/" + args["pool"],
                   lv_name=args["vol"],
                   options=["-T", "-V 10"],
                   command=lv_create,
                   errors=errors)

        atomic_run("Creating filesystem on LV.",
                   cmd="mkfs.ext4 /dev/%s/%s" % (args['group'], args['vol']),
                   command=run,
                   errors=errors)

        atomic_run("Deactivating thin LV.",
                   lv_name=args["vol"],
                   vg_name=args["group"],
                   command=lv_deactivate,
                   errors=errors)

    elif iteration == 3:
        for i in range(2):
            atomic_run("Creating thin LV No. %s." % i,
                       vg_name=args["group"] + "/" + args["pool"],
                       lv_name=args["vol"] + str(i),
                       options=["-T", "-V 10"],
                       command=lv_create,
                       errors=errors)

            atomic_run("Deactivating thin LV No. %s." % i,
                       lv_name=args["vol"] + str(i),
                       vg_name=args["group"],
                       command=lv_deactivate,
                       errors=errors)

    elif iteration == 4:
        for i in range(2):
            atomic_run("Creating thin LV No. %s." % i,
                       vg_name=args["group"] + "/" + args["pool"],
                       lv_name=args["vol"] + str(i),
                       options=["-T", "-V 10"],
                       command=lv_create,
                       errors=errors)
            if i == 1:
                atomic_run("Creating filesystem on LV No. %s." % i,
                           cmd="mkfs.ext4 /dev/%s/%s" % (args['group'], args['vol'] + str(i)),
                           command=run,
                           errors=errors)

            atomic_run("Deactivating thin LV No. %s." % i,
                       lv_name=args["vol"] + str(i),
                       vg_name=args["group"],
                       command=lv_deactivate,
                       errors=errors)

    elif iteration == 5:
        for i in range(2):
            atomic_run("Creating thin LV No. %s." % i,
                       vg_name=args["group"] + "/" + args["pool"],
                       lv_name=args["vol"] + str(i),
                       options=["-T", "-V 10"],
                       command=lv_create,
                       errors=errors)
            if i == 2:
                atomic_run("Creating filesystem on LV No. %s." % i,
                           cmd="mkfs.ext4 /dev/%s/%s" % (args['group'], args['vol'] + str(i)),
                           command=run,
                           errors=errors)

            atomic_run("Deactivating thin LV No. %s." % i,
                       lv_name=args["vol"] + str(i),
                       vg_name=args["group"],
                       command=lv_deactivate,
                       errors=errors)

    elif iteration == 6:
        for i in range(2):
            atomic_run("Creating thin LV No. %s." % i,
                       vg_name=args["group"] + "/" + args["pool"],
                       lv_name=args["vol"] + str(i),
                       options=["-T", "-V 10"],
                       command=lv_create,
                       errors=errors)

            atomic_run("Creating filesystem on LV No. %s." % i,
                       cmd="mkfs.ext4 /dev/%s/%s" % (args['group'], args['vol'] + str(i)),
                       command=run,
                       errors=errors)

            atomic_run("Deactivating thin LV No. %s." % i,
                       lv_name=args["vol"] + str(i),
                       vg_name=args["group"],
                       command=lv_deactivate,
                       errors=errors)


def thin_init(args, iteration):
    print("INFO: Initializing test case, iteration No. %s" % iteration)
    errors = []

    atomic_run("Creating loopdev",
               name=args["loop"],
               size=args["loop_size"],
               command=create_loopdev,
               errors=errors)

    atomic_run("Creating VG",
               vg_name=args["group"],
               pv_name="/dev/" + args["loop"],
               command=vg_create,
               errors=errors)

    atomic_run("Creating thin pool",
               vg_name=args["group"],
               lv_name=args["pool"],
               options=["-T", "-L 100"],
               command=lv_create,
               errors=errors)

    # Create LVs each time a bit different to cover all the conditions
    create_lvs(args, iteration, errors)

    atomic_run("Deactivating pool",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Creating swap LV",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-L 100"],
               command=lv_create,
               errors=errors)

    atomic_run("Deactivating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Swapping metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--thinpool " + args["group"] + "/" + args["pool"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Activating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Initialization No. %s passed" % iteration)
    else:
        TC.tfail("Initialization No. %s failed with following errors: \n\t'" % iteration +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def cache_init(args):
    print("INFO: Initializing test case.")
    errors = []

    atomic_run("Creating loopdev 1 - 'fast' device",
               name=args["loop"],
               size=args["loop_size"],
               command=create_loopdev,
               errors=errors)

    atomic_run("Creating loopdev 2 - 'slow' device",
               name=args["loop2"],
               size=args["loop2_size"],
               command=create_loopdev,
               errors=errors)

    atomic_run("Creating VG",
               vg_name=args["group"],
               pv_name="/dev/" + args["loop"] +
                       " /dev/" + args["loop2"],
               command=vg_create,
               errors=errors)

    atomic_run("Creating cache metadata volume",
               vg_name=args["group"] + " /dev/" + args["loop"],
               lv_name=args["meta"],
               options=["-L 12"],
               command=lv_create,
               errors=errors)

    atomic_run("Creating origin volume",
               vg_name=args["group"] + " /dev/" + args["loop2"],
               lv_name=args["origin"],
               options=["-L 2G"],
               command=lv_create,
               errors=errors)

    atomic_run("Creating cache data volume",
               vg_name=args["group"] + " /dev/" + args["loop"],
               lv_name=args["data"],
               options=["-L 1G"],
               command=lv_create,
               errors=errors)

    atomic_run("Creating cache pool",
               vg_name=args["group"],
               lv_name=args["data"],
               options=["-y --type cache-pool", "--cachemode writeback", "--poolmetadata %s/%s" %
                        (args["group"], args["meta"])],
               command=lv_convert,
               errors=errors)

    atomic_run("Creating cache logical volume",
               vg_name=args["group"],
               lv_name=args["origin"],
               options=["-y", "--type cache", "--cachepool %s/%s" %
                        (args["group"], args["data"])],
               command=lv_convert,
               errors=errors)

    atomic_run("Splitting cache logical volume",
               vg_name=args["group"],
               lv_name=args["origin"],
               options=["-y", "--splitcache"],
               command=lv_convert,
               errors=errors)

    atomic_run("Creating swap LV",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-L 100"],
               command=lv_create,
               errors=errors)

    atomic_run("Swapping metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--cachepool " + args["group"] + "/" + args["data"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Activating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Initialization passed")
    else:
        TC.tfail("Initialization failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def thin_clean(args):
    print("INFO: Cleaning up")
    errors = []

    # restoring metadata device after corrupting it
    atomic_run("Repairing metadata device",
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               quiet=True,
               command=thin_restore,
               errors=errors)

    # thinpool got activated after checking its metadata to get bad checksum
    atomic_run("Deactivating pool",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Deactivating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Swapping back metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--thinpool " + args["group"] + "/" + args["pool"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Removing VG",
               vg_name=args["group"],
               force=True,
               command=vg_remove,
               errors=errors)

    atomic_run("Deleting loopdev",
               name=args["loop"],
               command=delete_loopdev,
               errors=errors)

    atomic_run("Deleting metadata file",
               cmd="rm -f /var/tmp/metadata",
               command=run,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def cache_clean(args):
    print("INFO: Cleaning up")
    errors = []

    atomic_run("Removing VG",
               vg_name=args["group"],
               force=True,
               command=vg_remove,
               errors=errors)

    atomic_run("Deleting loopdev loop1",
               name=args["loop"],
               command=delete_loopdev,
               errors=errors)

    atomic_run("Deleting loopdev loop2",
               name=args["loop2"],
               command=delete_loopdev,
               errors=errors)

    atomic_run("Deleting metadata file",
               cmd="rm -f /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Deleting repair metadata file",
               cmd="rm -f /var/tmp/metadata_repair",
               command=run,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def install_packages(packages):
    for package in packages:
        install_package(package)


def thin(args, iteration):
    print("\n#######################################\n")
    print("INFO: Testing restoring corruption with thin_restore, iteration No. %s" % iteration)

    errors = []

    atomic_run("Dumping metadata to file",
               formatting="xml",
               source_vg=args["group"],
               source_lv=args["swap"],
               output="/var/tmp/metadata",
               command=thin_dump,
               errors=errors)

    atomic_run("Checking metadata",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_check,
               errors=errors)

    atomic_run("Restoring metadata from nonexistent file",
               False,
               cmd="thin_restore -i /var/tmp/wrong.wrong -o /dev/mapper/%s-%s" % (args['group'], args['swap']),
               command=run,
               errors=errors)

    # should pass without error on the last iteration
    if iteration != 6:
        atomic_run("Checking metadata",
                   pkg=["device-mapper-persistent-data", "0.7.3", "1.el7"],
                   source_vg=args["group"],
                   source_lv=args["swap"],
                   command=thin_check,
                   errors=errors)
    else:
        atomic_run("Checking metadata",
                   source_vg=args["group"],
                   source_lv=args["swap"],
                   command=thin_check,
                   errors=errors)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing restoring corruption with thin_restore passed, iteration No. %s" % iteration)
    else:
        TC.tfail("Testing restoring corruption with thin_restore failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]) +
                 "\nIteration No. %s" % iteration)
        return 1
    return 0


def cache(args):
    print("\n#######################################\n")
    print(
        "INFO: Testing restoring corruption with cache_restore.")

    errors = []

    atomic_run("Dumping metadata to file",
               source_vg=args["group"],
               source_lv=args["swap"],
               output="/var/tmp/metadata",
               command=cache_dump,
               errors=errors)

    atomic_run("Checking metadata",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_check,
               errors=errors)

    atomic_run("Restoring metadata with options",
               False,
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               omit_clean_shutdown=True,
               command=cache_restore,
               errors=errors)

    atomic_run("Restoring metadata from nonexistent file",
               False,
               cmd="cache_restore -i /var/tmp/wrong.wrong -o /dev/mapper/%s-%s" % (args['group'], args['swap']),
               command=run,
               errors=errors)

    # The fail here is caused by using "omit-clean-shotdown" before
    atomic_run("Checking metadata",
               pkg=["device-mapper-persistent-data", "0.7.3", "1.el7"],
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_check,
               errors=errors)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing restoring corruption with thin_restore passed")
    else:
        TC.tfail("Testing restoring corruption with thin_restore failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    args = {"loop": "loop1",
            "loop_size": 2048,
            "loop2": "loop2",
            "loop2_size": 4128,
            "group": "vgtest",
            "pool": "thinpool",
            "vol": "thinvol",
            "number of vols": 10,
            "origin": "origin",
            "data": "cache_data",
            "meta": "cache_meta",
            "swap": "swapvol"}

    # Initialization
    packages = ["device-mapper-persistent-data"]
    install_packages(packages)

    # thin_restore testing
    for i in range(1, 7):
        thin_init(args, i)
        thin(args, i)
        thin_clean(args)

    # cache_restore testing
    cache_init(args)
    cache(args)
    cache_clean(args)

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, read_env, get_func_from_string
from stqe.host.persistent_vars import write_var, read_var
from stqe.host.stratis import get_stratis_args
from libsan.host.stratis import Stratis


def write_data(args, errors):
    name = read_env("fmf_name")
    if "fs_create" in name:
        atomic_run("Writing var STRATIS_FS",
                   command=write_var,
                   var={'STRATIS_FS': args['fs_name']},
                   errors=errors)

    if "snapshot_create" in name:
        atomic_run("Writing var STRATIS_SNAPSHOT",
                   command=write_var,
                   var={'STRATIS_SNAPSHOT': args['snapshot_name']},
                   errors=errors)

    if "mkdir" in name:
        atomic_run("Writing var FS_DIR",
                   command=write_var,
                   var={'FS_DIR': args['cmd'].split()[-1]},
                   errors=errors)
    return


def setup():
    errors = []

    stratis = Stratis(disable_check=True)
    args = get_stratis_args(stratis_object=stratis)
    print(args)

    # some extra args for some setup
    args.update(get_env_args(["id", "cmd", "path"],
                             read_var("FILE_NAMES")))
    command = args.pop('command')

    args['command'] = get_func_from_string(stratis, command)
    atomic_run(errors=errors,
               **args)
    print("the second args %s" % args)
    write_data(args, errors)

    return errors


if __name__ == "__main__":
    errs = setup()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_create_success():
    errors = []

    fs_name = read_var("LSM_FS_NAME")
    fs_size = read_var("LSM_FS_SIZE")
    pool_id = read_var("LSM_POOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Creating FS %s with protocol %s." % (fs_name, config['protocol']),
                               command=lsm.fs_create,
                               name=fs_name,
                               size=fs_size,
                               pool=pool_id,
                               return_output=True,
                               errors=errors)
        fs_id = [line.split()[0].strip() for line in data.splitlines() if pool_id in line][0]
        atomic_run("Removing FS %s with protocol %s." % (fs_name, config['protocol']),
                   command=lsm.fs_delete,
                   fs=fs_id,
                   force=True,
                   errors=errors)
    return errors


def fs_create_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")
    fs_size = read_var("LSM_FS_SIZE")
    pool_id = read_var("LSM_POOL_ID")
    pool_name = read_var("LSM_POOL_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail creating FS without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.fs_create},
            {'message': "Trying to fail creating FS without name with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': fs_size, 'pool': pool_id},
            {'message': "Trying to fail creating FS without size with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'name': fs_name, 'pool': pool_id},
            {'message': "Trying to fail creating FS without pool with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': fs_size, 'name': fs_name},
            {'message': "Trying to fail creating FS with wrong size '-1' with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': -1, 'name': fs_name, 'pool': pool_id},
            {'message': "Trying to fail creating FS with wrong size '0' with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': 0, 'name': fs_name, 'pool': pool_id},
            {'message': "Trying to fail creating FS with wrong size 'wrong' with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': "wrong", 'name': fs_name, 'pool': pool_id},
            {'message': "Trying to fail creating FS with pool 'NONEXISTENT' with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': fs_size, 'name': fs_name, 'pool': "NONEXISTENT"},
            {'message': "Trying to fail creating FS with pool name instead of ID with protocol %s" % config['protocol'],
             'command': lsm.fs_create, 'size': fs_size, 'name': fs_name, 'pool': pool_name},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def fs_create_fail_name_conflict():
    errors = []

    fs_name = read_var("LSM_FS_NAME")
    fs_size = read_var("LSM_FS_SIZE")
    pool_id = read_var("LSM_POOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Creating FS %s again to hit NAME CONFLICT with protocol %s." % (fs_name, config['protocol']),
                   command=lsm.fs_create,
                   name=fs_name,
                   size=fs_size,
                   pool=pool_id,
                   expected_ret=4,
                   errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_create_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_create_fail()
        errs += fs_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.scsi as scsi
import libsan.host.iscsi as iscsi
import libsan.host.mp as mp
import libsan.host.fc as fc
import libsan.host.net as net
import libsan.sanmgmt as sanmgmt
import libsan.host.linux as linux
import libsan.host.dt as dt
import libsan.misc.array as array
import stqe.host.nfs_lock as nfs_lock
import stqe.host.LogChecker as log
import libsan.misc.time
import re
import sys
import os
import argparse

obj_sanmgmt = None

opt_failover_mode = None
opt_max_interval = None
opt_max_flap_uptime = None
opt_max_flap_downtime = None
opt_max_flap_count = None
opt_dt_time = None

all_supported_failover_methods = ["sa_ctrler_reboot", "fc_target",
                                  "physw_flap_target", "physw_target",
                                  "physw_flap_host", "physw_host",
                                  "iscsi_session", "sysfs",
                                  "fc_host", "iscsi_host"]

_check_method_to_func = {}


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _check_failover_iscsi_session(mpath):
    """
    """
    if not mpath:
        _print("FAIL: _check_failover_iscsi_session() - requires mpath_name parameter")
        return None

    _print("DEBUG: _check_failover_iscsi_session() - mpath %s" % mpath)
    mp_info = mp.multipath_query_all(mpath)

    if "disk" not in mp_info:
        _print("FAIL: _check_failover_iscsi_session() - %s has no SCSI disk" % mpath)
        return None

    iscsi_sessions = iscsi.query_all_iscsi_sessions()
    if not iscsi_sessions:
        return None
    failover_option_list = []
    for scsi_disk in mp_info["disk"].keys():
        for ses in iscsi_sessions.keys():
            if "disks" not in iscsi_sessions[ses]:
                continue
            if scsi_disk in iscsi_sessions[ses]["disks"]:
                iface = iscsi_sessions[ses]["iface"]
                t_iqn = iscsi_sessions[ses]["t_iqn"]
                persist_ip = iscsi_sessions[ses]["persist_ip"]
                failover_option = "-T %s -p %s -I %s" % (t_iqn, persist_ip, iface)
                failover_option_list.append(failover_option)

    failover_option_list = array.dedup(failover_option_list)
    if len(failover_option_list) < 2:
        _print("WARN: There is less than 2 iSCSI sessions running on %s" % mpath)
        return None
    return failover_option_list


def _check_failover_sysfs(mpath_name):
    """
    Usage
      check_failover_sysfs(mpath_name)
    Purpose
      Check whether all the disks has this sysfs file:
          /sys/block/sdX/device/state
    Parameter
      mpath_name     # like 'mpathb'
    Returns
      failover_entries
          or
      None
    """
    if not mpath_name:
        _print("FAIL: _check_failover_sysfs() - requires mpath_name parameter")
        return None

    _print("DEBUG: _check_failover_sysfs() - mpath %s" % mpath_name)
    disks = mp.get_disks_of_mpath(mpath_name)
    if not disks:
        return None

    if len(disks) < 2:
        _print("FAIL: %s has less then 2 paths, we won't run sysfs failover" % mpath_name)
        print(disks)
        return None

    failure = False
    for disk in disks:
        path = "/sys/block/%s/device/state" % disk
        if not os.path.isfile(path):
            failure = True
            _print("FAIL: %s  has not sysfs file: %s" % (disk, path))

    if failure:
        _print("FAIL: check_failover_sysfs(): Multipath %s has some disk "
               + "does not have /sys/block/sdX/device/state file" % mpath_name)
        return None

    return disks


def _check_failover_fc_host(mpath_name):
    """
    Usage
      check_failover_fc_host(mpath_name)
    Purpose
      Check whether switches which connected HBA ports is controlable.
    Parameter
      mpath_name     # like 'mpathb'
    Returns
      failover_entries
          or
      None
    """
    if ("FC" not in mp.transport_types_of_mpath(mpath_name) and
            "FCoE" not in mp.transport_types_of_mpath(mpath_name)):
        return None

    return _check_failover_switch(mpath_name=mpath_name,
                                  flag_host=True, flag_physw=False)


def _check_failover_fc_target(mpath_name):
    """
    Usage
      _check_failover_fc_target(mpath_name)
    Purpose
      Check whether switches which connected Target port is controlable.
    Parameter
      mpath_name     # like 'mpathb'
    Returns
      failover_entries
          or
      None
    """
    if ("FC" not in mp.transport_types_of_mpath(mpath_name) and
            "FCoE" not in mp.transport_types_of_mpath(mpath_name)):
        return None

    return _check_failover_switch(mpath_name=mpath_name,
                                  flag_host=False, flag_physw=False)


def _check_failover_physw_host(mpath_name):
    """
    Usage
      check_failover_physw_host(mpath_name)
    Purpose
      Check whether physical switch which connected HBA ports is controlable.
    Parameter
      mpath_name     # like 'mpathb'
    Returns
      failover_entries
          or
      None
    """
    return _check_failover_switch(mpath_name=mpath_name,
                                  flag_host=True, flag_physw=True)


def _check_failover_physw_target(mpath_name):
    """
    Usage
      _check_failover_physw_target(mpath_name)
    Purpose
      Check whether physical switches which connected Target port is controlable.
    Parameter
      mpath_name     # like 'mpathb'
    Returns
      failover_entries
          or
      None
    """
    return _check_failover_switch(mpath_name=mpath_name,
                                  flag_host=False, flag_physw=True)


def _check_failover_iscsi_host(mpath_name):
    """
    Usage
      check_failover_iscsi_host(mpath_name)
    Purpose
      Check whether switches which connected HBA ports is controlable.
    Parameter
      mpath_name     # like 'mpathb'
    Returns
      failover_entries
          or
      None
    """
    if "iSCSI" not in mp.transport_types_of_mpath(mpath_name):
        return None

    return _check_failover_switch(mpath_name=mpath_name,
                                  flag_host=True, flag_physw=False)


def _check_failover_sa_ctrler_reboot(mpath_name):
    """
    """
    global obj_sanmgmt

    if not mpath_name:
        _print("FAIL: _check_failover_sa_ctrler_reboot() - requires mpath_name parameter")
        return None

    mp_t_wwpns = mp.t_wwpns_of_mpath(mpath_name)
    if not mp_t_wwpns:
        _print("FAIL: _check_failover_sa_ctrler_reboot() - Did not find target WWPN for %s" % mpath_name)
        return None

    sa_t_wwpn_2_ctrler = obj_sanmgmt.sa_t_wwpn_2_ctrler()
    if not sa_t_wwpn_2_ctrler:
        _print("FAIL: _check_failover_sa_ctrler_reboot() - Did not find target WWPN on target controllers")
        return None

    if len(sa_t_wwpn_2_ctrler) < 2:
        _print("FAIL: _check_failover_sa_ctrler_reboot() - Storage array has only 1 controller")
        print(sa_t_wwpn_2_ctrler)
        return None

    t_wwpns = []
    mp_ctrlers = []  # ctrlers used by mpath
    for ctrler in sa_t_wwpn_2_ctrler.keys():
        for t_wwpn in sa_t_wwpn_2_ctrler[ctrler]:
            if t_wwpn not in mp_t_wwpns:
                # Skip t_wwpn if it is not used by multipath device
                continue
            if t_wwpn not in t_wwpns:
                t_wwpns.append(t_wwpn)
            if ctrler not in mp_ctrlers:
                mp_ctrlers.append(ctrler)

    if len(mp_ctrlers) < 2:
        _print(
            "FAIL: _check_failover_sa_ctrler_reboot() - mpath %s is connected to less then 2 controllers" % mpath_name)
        return None

    if len(t_wwpns) < 2:
        _print(
            "FAIL: _check_failover_sa_ctrler_reboot() - "
            "Despite having more then 1 controller, we found less then 2 target WWPNs")
        print(t_wwpns)
        return None

    return t_wwpns


# This dict should be defined after all _check_failover'method' functions are defnied
_check_method_to_func["iscsi_session"] = _check_failover_iscsi_session
_check_method_to_func["iscsi_host"] = _check_failover_iscsi_host
_check_method_to_func["sysfs"] = _check_failover_sysfs
_check_method_to_func["fc_host"] = _check_failover_fc_host
_check_method_to_func["fc_target"] = _check_failover_fc_target
_check_method_to_func["physw_host"] = _check_failover_physw_host
_check_method_to_func["physw_target"] = _check_failover_physw_target
_check_method_to_func["physw_flap_host"] = _check_failover_physw_host
_check_method_to_func["physw_flap_target"] = _check_failover_physw_target
_check_method_to_func["sa_ctrler_reboot"] = _check_failover_sa_ctrler_reboot


def _check_failover_switch(mpath_name=None,
                           flag_host=True, flag_physw=False):
    """
    Usage
      _check_failover_switch(mpath_name=>$mpath_name,
          flag_physw=>$flag_physw, flag_host=>$flag_host)
    Purpose
      Check SanMgmt for failover on swtich and physical switch.
    Parameter
      mpath_name     # like "mpatha"
      flag_physw     # physwitch or not
      flag_host      # host level failover or not
    """
    global obj_sanmgmt

    if not mpath_name:
        return None

    port_string = "Storage Array port"
    failover_type_string = "target"
    if flag_host:
        port_string = "HBA port"
        failover_type_string = "host"
        if "iSCSI" in mp.transport_types_of_mpath(mpath_name):
            ports = mp.iface_macs_of_mpath(mpath_name)
        else:
            ports = mp.h_wwpns_of_mpath(mpath_name)
    else:
        ports = mp.t_wwpns_of_mpath(mpath_name)

    switch_string = "switch"
    method_action_up = "link_up"
    method_action_down = "link_down"
    if flag_physw:
        switch_string = "physical switch"
        method_action_up = "phy_port_connect"
        method_action_down = 'phy_port_disconnect'

    if not ports:
        _print("WARN: Multipath %s has no %s " % (mpath_name, failover_type_string)
               + "WWPN")
        _print("WARN: Can't run %s %s level failover on %s" %
               (switch_string, failover_type_string, mpath_name))
        return None

    if len(ports) < 2:
        _print("WARN: Multipath %s only 1 %s " % (mpath_name, failover_type_string)
               + "WWPN")
        _print("WARN: Can't run %s %s level failover on %s" %
               (switch_string, failover_type_string, mpath_name))
        return None

    capability = obj_sanmgmt.capability()
    if not capability:
        _print("FAIL: SanMgmt could not find any capability")
        return None

    if ((method_action_up not in capability) or
            (method_action_down not in capability) or
            not capability[method_action_up] or
            not capability[method_action_down]):
        _print("WARN: SanMgmt cannot do %s/%s on %s" %
               (method_action_up, method_action_down, switch_string))
        print(capability)
        return None

    ready_ports = []
    for port in ports:
        if port not in capability[method_action_down]:
            _print("WARN: SanMgmt cannot do %s on %s port(%s)" %
                   (method_action_down, port_string, port))
            continue
        if port not in capability[method_action_up]:
            _print("WARN: SanMgmt cannot do %s on %s port(%s)" %
                   (method_action_up, port_string, port))
            continue
        ready_ports.append(port)

    if not ready_ports:
        _print("WARN: None of %s Ports can do failover at %s level" %
               (port_string, switch_string))
        return None

    return ready_ports


def check_failover_methods(mpath):
    """
    """
    global opt_failover_mode, all_supported_failover_methods, _check_method_to_func
    if not mpath:
        _print("FAIL: check_failover_methods() - requires mpath parameter")
        return None

    if opt_failover_mode:
        all_failover_methods = re.split(",| ", opt_failover_mode)
        # remove any "empty string that might have been added"
        while "" in all_failover_methods:
            all_failover_methods.remove("")
    else:
        all_failover_methods = all_supported_failover_methods

    failover_method_2_entries = None
    for method in all_failover_methods:
        # Check if mpath supports this failover
        if method in _check_method_to_func:
            failover_entries = _check_method_to_func[method](mpath)
            if failover_entries:
                if not failover_method_2_entries:
                    failover_method_2_entries = {}
                failover_method_2_entries[method] = failover_entries

    return failover_method_2_entries


###############################################################################

def entry_down(entry, failover_method):
    """
    """
    return entry_trigger(entry=entry, failover_method=failover_method, action="DOWN")


def entry_up(entry, failover_method):
    """
    """
    ret = entry_trigger(entry=entry, failover_method=failover_method, action="UP")
    _print("INFO: Waiting 10 seconds for system to detect the revive")
    linux.sleep(10)
    return ret


def entry_trigger(entry=None, failover_method=None, action=None):
    """
    """
    global opt_max_flap_uptime, opt_max_flap_downtime, opt_max_flap_count
    global obj_sanmgmt

    if not entry or not failover_method or not action:
        return False

    if action != "UP" and action != "DOWN":
        _print("FAIL: action %s is not supported by entry_trigger" % action)
        return False

    # _print("DEBUG: Executing %s action %s on %s" % (failover_method, action, entry))

    if failover_method == "sysfs":
        _print("INFO: bringing %s %s via sysfs" % (entry, action))
        return scsi.disk_sys_trigger(entry, action)

    if (failover_method == "fc_host") or (failover_method == "fc_target"):
        _print("INFO: bringing %s %s via %s" % (entry, action, failover_method))
        return obj_sanmgmt.link_trigger(action=action, wwpn=entry)

    if (failover_method == "physw_host") or (failover_method == "physw_target"):
        _print("INFO: bringing %s %s via %s" % (entry, action, failover_method))
        return obj_sanmgmt.phy_link_trigger(action=action, addr=entry)

    if (failover_method == "physw_flap_host") or (failover_method == "physw_flap_target"):
        _print("INFO: Flapping %s via %s" % (entry, failover_method))
        return obj_sanmgmt.phy_link_flap(entry, opt_max_flap_uptime,
                                         opt_max_flap_downtime, opt_max_flap_count)

    if failover_method == "iscsi_host":
        _print("INFO: bringing %s %s via %s" % (entry, action, failover_method))
        return obj_sanmgmt.link_trigger(action=action, addr=entry)

    if failover_method == "iscsi_session":
        _print("INFO: bringing %s %s via %s" % (entry, action, failover_method))
        if action == "UP":
            return iscsi.node_login(entry)
        elif action == "DOWN":
            return iscsi.node_logout(entry)
        else:
            _print("FAIL: unsupported action %s for %s" % (action, failover_method))
            return False

    if failover_method == "sa_ctrler_reboot":
        _print("INFO: bringing controller on port %s %s via %s" % (entry, action, failover_method))
        if action == "UP":
            return obj_sanmgmt.sa_ctrler_wait(entry)
        elif action == "DOWN":
            return obj_sanmgmt.sa_ctrler_reboot(entry)
        else:
            _print("FAIL: unsupported action %s for %s" % (action, failover_method))
            return False

    _print("FAIL: unsupported failover_method %s" % failover_method)
    return False


def _get_scsi_devices_by_entry(mpath, entry):
    """
    Return as list of SCSI devices based on entry
    """
    if not mpath or not entry:
        _print("FAIL: _get_scsi_devices_by_entry() - requires mpath and entry as parameter")
        return None
    if scsi.is_scsi_device(entry):
        return [entry]
    if fc.standardize_wwpn(entry):
        return mp.get_disks_of_mpath_by_wwpn(mpath, entry)
    if net.is_mac(entry):
        return mp.get_disks_of_mpath_by_mac(mpath, entry)
    # For example on iSCSI entry contains target iqn, intercace and portal info:
    _print("WARN: Could not get mpath for entry (%s), returning all SCSI devices from %s" % (entry, mpath))
    return mp.get_disks_of_mpath(mpath)


def execute_failover(mpath=None, failover_method_2_entries=None, flag_quit_on_mp_error=True, dt_pid=None, runtime=None):
    """
    execute_failover ()
    Usage
      execute_failover(mpath=>$mpath_name,
          failover_method_2_entries =failover_method_2_entries,
          dt_pid=dt_pid,
          quit_on_mp_error=>flag_quit_on_mp_error,
          )
    Purpose
      Execute failover in many ways.
    Parameter
      mpath_name             # like "mpathb"
      failover_method_2_entries # return reference of check_failover_methods()
      dt_pid                 # child progress ID which runnig background dt.
      flag_check_mp          # check multipath output or not
      flag_quit_on_mp_error  # quit or not if multipath failed to maintain info
    Returns
      True                       # DT pass and all link failover pass
          or
      False
    """
    global opt_max_interval

    # Test if DT process will finish within its runtime...
    runtime_sec = libsan.misc.time.time_2_sec(runtime)
    runtime_sec += 60  # give 1 minute of tolerance for dt to finish
    expire_time = libsan.misc.time.get_time(in_seconds=True)
    expire_time += runtime_sec

    if not mpath or not failover_method_2_entries:
        _print("FAIL: execute_failover() - requires mpath and failover_method_2_entries parameters")
        return False

    # flag_dt_done = False

    # Run until DT process is over
    flag_dt_done, exit_status = os.waitpid(dt_pid, os.WNOHANG)
    while flag_dt_done == 0:
        for failover_method in failover_method_2_entries:
            # Most of the tests cause some disk from multipath to go down
            # but sa_ctrler_reboot does not as NetApp controller, will fallback
            # To other controller
            # And link flap also does not bring the link down for too long
            expect_disk_down = True
            if (failover_method == "sa_ctrler_reboot" or
                    failover_method == "physw_flap_host" or
                    failover_method == "physw_flap_target"):
                expect_disk_down = False
            _print("\n\nINFO: ==== Executing failover at %s level." % failover_method)

            failover_entries = failover_method_2_entries[failover_method]
            if not failover_entries:
                _print("FAIL: No failover entries available for %s" % failover_method)
                return False

            for entry in failover_entries:
                run("multipath -ll %s" % mpath)

                if dt_pid:
                    flag_dt_done, exit_status = os.waitpid(dt_pid, os.WNOHANG)
                    if flag_dt_done:
                        # DT finished, we can stop failover
                        if exit_status == 0:
                            return True
                        return False
                    # DT did not finish yet, check if it got stuck...
                    if libsan.misc.time.get_time(in_seconds=True) > expire_time:
                        _print("FAIL: DT process should have finished by now, but it did not...")
                        run("ps -ef | grep dt")
                        return False

                pre_running_paths = []
                scsi_disks = mp.get_disks_of_mpath(mpath)
                if not scsi_disks:
                    _print("FAIL: Could not find any SCSI disk for %s" % mpath)
                    return False

                for scsi_disk in scsi_disks:
                    if mp.mpath_check_disk_status(mpath, scsi_disk) == "running":
                        pre_running_paths.append(scsi_disk)
                # running_paths_cmd = "multipathd show paths format '%d, %o, %m' "
                # running_paths_cmd += "| grep -w %s | grep -w running" % mpath
                # ret, output = run(running_paths_cmd, return_output=True)
                # pre_running_paths = len(output.split("\n"))
                # _print("DEBUG: execute_failover() - pre running paths %d" % len(pre_running_paths))

                scsi_devices_port_up = _get_scsi_devices_by_entry(mpath, entry)

                if not scsi_devices_port_up or (len(scsi_devices_port_up) == 0):
                    _print("FAIL: Could not find any SCSI device for: %s" % entry)
                    return False

                _print("\nINFO: Trying to bring %s DOWN via %s" % (entry, failover_method))
                down_ok = entry_down(entry, failover_method)
                if not down_ok:
                    _print("FAIL: Failed to bring %s (%s) DOWN" % (entry, failover_method))
                    entry_up(entry, failover_method)
                    return False
                _print("INFO: Successfully brought %s down via %s" % (entry, failover_method))

                _print("INFO: Interval %d seconds" % opt_max_interval)
                linux.sleep(opt_max_interval)

                run("multipath -ll %s" % mpath)
                sleep_time = 60
                _print("INFO: we wait %ds for the multipathd to get the correct path status" % sleep_time)
                linux.sleep(sleep_time)
                run("multipath -ll %s" % mpath)

                # confirm the multipath detect the disk state change
                # if entry is scsi_device we check it
                # if entry is a WWPN/IQN, we try to find the disks connected to this port
                scsi_devices_port_down = _get_scsi_devices_by_entry(mpath, entry)

                # As the port is down, it is expected to not have any scsi_device connected to it
                # If there is any, they should not be on "running" status
                found_disk_down = False
                if scsi_devices_port_down:
                    for device in scsi_devices_port_down:
                        _print("INFO: Checking status of: %s that is connected to %s" % (device, entry))
                        disk_status = mp.mpath_check_disk_dm_status(mpath, device)
                        # If disk status is None, it is because OS removed the offline device
                        if not disk_status or "active" != disk_status:
                            found_disk_down = True
                        disk_status = mp.mpath_check_disk_path_status(mpath, device)
                        # If disk status is None, it is because OS removed the offline device
                        if not disk_status or "ready" != disk_status:
                            found_disk_down = True

                    # maybe the scsi disk is removed from the path,
                    #  in that case scsi_devices will have less disks than scsi_disks
                    if len(scsi_devices_port_down) < len(scsi_devices_port_up):
                        found_disk_down = True
                else:
                    found_disk_down = True

                if expect_disk_down and not found_disk_down:
                    _print("FAIL: Multipath did not detect disk DOWN")
                    _print("INFO: Restoring %s back up" % entry)
                    entry_up(entry, failover_method)
                    if flag_quit_on_mp_error:
                        return False

                _print("\nINFO: Trying to bring %s UP via %s" % (entry, failover_method))
                up_ok = entry_up(entry, failover_method)
                if not up_ok:
                    _print("FAIL: Failed to bring %s (%s) UP" % (entry, failover_method))
                    return False
                _print("INFO: Successfully brought %s up via %s" % (entry, failover_method))

                run("multipath -ll %s" % mpath)

                _print("INFO: we wait %ds for the multipathd to get the correct path status" % sleep_time)
                linux.sleep(sleep_time)
                run("multipath -ll %s" % mpath)
                # confirm the multipath detect the disk state change
                if scsi.is_scsi_device(entry):
                    scsi_devices = [entry]
                else:
                    if fc.standardize_wwpn(entry):
                        scsi_devices = mp.get_disks_of_mpath_by_wwpn(mpath, entry)
                    else:
                        # For example on iSCSI entry contains target iqn, intercace and portal info:
                        scsi_devices = mp.get_disks_of_mpath(mpath)

                if not scsi_devices:
                    _print("FAIL: Could not get scsi disks UP for %s" % entry)
                    return False

                for device in scsi_devices:
                    disk_status = mp.mpath_check_disk_dm_status(mpath, device)
                    if not disk_status:
                        _print("FAIL: Could not get disk status for %s with %s UP" % (device, entry))
                        _print("INFO: Restoring %s back up" % entry)
                        entry_up(entry, failover_method)
                        if flag_quit_on_mp_error:
                            return False

                    if "active" != disk_status:
                        _print("FAIL: Multipath did not detect disk UP")
                        _print("FAIL: Multipath is reporting the disk with status %s" % disk_status)
                        run("multipath -ll %s" % mpath)
                        _print("INFO: Restoring %s back up" % entry)
                        entry_up(entry, failover_method)
                        if flag_quit_on_mp_error:
                            return False

                post_running_paths = []
                scsi_disks = mp.get_disks_of_mpath(mpath)
                if not scsi_disks:
                    _print("FAIL: Could not find any SCSI disk for %s" % mpath)
                    return False

                for scsi_disk in scsi_disks:
                    if mp.mpath_check_disk_status(mpath, scsi_disk) == "running":
                        post_running_paths.append(scsi_disk)

                # _print("DEBUG: execute_failover() - post running paths %d" % len(post_running_paths))
                if len(pre_running_paths) != len(post_running_paths):
                    _print("FAIL: Different amount of paths after failover")
                    print("Before failover: ", pre_running_paths)
                    print("After failover: ", post_running_paths)
                    return False

        if dt_pid:
            flag_dt_done, exit_status = os.waitpid(dt_pid, os.WNOHANG)
        else:
            # No PID for DT was given, we run just once
            flag_dt_done = 1
        # end while flag_dt_done

    if exit_status == 0:
        return True
    return False


def execute_test(mpath_name):
    global obj_sanmgmt

    mp.multipath_show(mpath_name)

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Could not create SanMgmt obj for %s" % mpath_name)
        sys.exit(1)

    # Let get lock for this Storage array
    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if not lock_obj:
        _print("WARN: There is no NFS lock configured!!!")
    lock_type = "exclusive"
    # Tests that do not reboot controller neither change Storage Array port on
    # switch can share the array
    if (opt_failover_mode and (opt_failover_mode == "fc_host"
                               or opt_failover_mode == "physw_flap_host"
                               or opt_failover_mode == "physw_host"
                               or opt_failover_mode == "sysfs"
                               or opt_failover_mode == "iscsi_host"
                               or opt_failover_mode == "iscsi_session")):
        lock_type = "shared"

    # need to request the lock before checking ports, otherwise some port might be down
    # due other server executing tests
    if lock_obj:
        _print("INFO: Requesting NFS lock (%s)..." % lock_type)
        if not lock_obj.request_lock(lock_type):
            _print("FAIL: Could not request NFS lock")
            sys.exit(1)

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            sys.exit(1)
        _print("INFO: Good, got NFS lock. Start testing...")

    # make sure we have the ports up before querying multipath info
    if not obj_sanmgmt.check_ports_ready():
        _print("FAIL: Not all ports that we need are UP")
        if lock_obj:
            _print("INFO: Releasing for NFS lock...")
            lock_obj.release_lock(lock_type)
        sys.exit(1)

    mpath_info = mp.multipath_query_all(mpath_name)
    if not mpath_info:
        _print("FAIL: Could not get info for mpath: %s" % mpath_name)
        if lock_obj:
            _print("INFO: Releasing for NFS lock...")
            lock_obj.release_lock(lock_type)
        sys.exit(1)

    failover_method_2_entries = check_failover_methods(mpath_name)

    if not failover_method_2_entries:
        _print("SKIP: No failover method available for %s. SKIP this test" % mpath_name)
        if lock_obj:
            _print("INFO: Releasing for NFS lock...")
            lock_obj.release_lock(lock_type)
        sys.exit(2)

    # print mpath_info
    # print failover_method_2_entries
    for failover_method in failover_method_2_entries.keys():
        _print("INFO: %s will perform \"%s\" failover on these entries" % (mpath_name, failover_method))
        for entries in failover_method_2_entries[failover_method]:
            _print("%4s%s" % (" ", entries))

    dt_log_file = "/tmp/dt_stress_mp_failover.log"
    pid = dt.dt_stress_background(of="/dev/mapper/%s" % mpath_name, log=dt_log_file, time=opt_dt_time)
    if not pid:
        _print("FAIL: Could not start DT on background")
        if lock_obj:
            _print("INFO: Releasing for NFS lock...")
            lock_obj.release_lock(lock_type)
        sys.exit(1)

    ok = execute_failover(mpath=mpath_name, failover_method_2_entries=failover_method_2_entries, dt_pid=pid,
                          runtime=opt_dt_time)

    # If for some reason we left some port DOWN, try to bring them UP again
    obj_sanmgmt.check_ports_ready()

    if lock_obj:
        _print("INFO: Releasing for NFS lock...")
        lock_obj.release_lock(lock_type)

    if ok:
        if not log.check_all():
            _print("FAIL: detected error on LogChecker")
            if lock_obj:
                _print("INFO: Releasing for NFS lock...")
                lock_obj.release_lock(lock_type)
            sys.exit(1)

        if lock_obj:
            _print("INFO: Releasing for NFS lock...")
            lock_obj.release_lock(lock_type)
        sys.exit(0)
    else:
        if linux.check_pid(pid):
            _print("FAIL: There was some problem while running execute_failover")
            linux.kill_all("dt")
        else:
            _print("FAIL: Got data corruption when DT I/O stress. Dumping DT log")
            run("cat %s" % dt_log_file)
        if not log.check_all():
            _print("FAIL: detected error on LogChecker")
        if lock_obj:
            _print("INFO: Releasing for NFS lock...")
            lock_obj.release_lock(lock_type)
        sys.exit(1)


def main():
    global opt_failover_mode, opt_max_interval
    global opt_dt_time, opt_max_flap_uptime, opt_max_flap_downtime, opt_max_flap_count

    PASS = 0
    FAIL = 1

    parser = argparse.ArgumentParser(
        description='mp_failover')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to use (We use it to create new device based on it)",
                        metavar="mpath")
    parser.add_argument("--dt-time", required=False, dest="dt_time",
                        default="1h", help="How long should DT run for", metavar="time")
    parser.add_argument("--max-interval", required=False, dest="max_interval",
                        default=1, type=int, help=":")
    parser.add_argument("--quit-on-mp-error", required=False, dest="quit_on_mp_error",
                        help=":")
    parser.add_argument("--max-flap-up-time", required=False, dest="max_flap_uptime",
                        default=100, type=int,
                        help="The max up time in microseconds for physical switch to bring link down when flaping.")
    parser.add_argument("--max-flap-down-time", required=False, dest="max_flap_downtime",
                        default=100, type=int,
                        help="The max down time in microseconds for physical switch to bring link down when flaping.")
    parser.add_argument("--max-flap-count", required=False, dest="max_flap_count",
                        default=50, type=int,
                        help="The max flap count for physical switch to torture the link")
    parser.add_argument("--failover-mode", required=False, dest="failover_mode",
                        help="Specify the only failover mode to run. Current support failover modes:" +
                             "sysfs\n" +
                             "fc_host\n" +
                             "fc_target\n" +
                             "physw_host\n" +
                             "physw_target\n" +
                             "physw_flap_host\n" +
                             "physw_flap_target\n" +
                             "iscsi_session\n" +
                             "sa_ctrler_reboot", metavar="mode_name")

    args = parser.parse_args()

    opt_failover_mode = args.failover_mode
    opt_dt_time = args.dt_time
    opt_max_interval = args.max_interval
    opt_max_flap_uptime = args.max_flap_uptime
    opt_max_flap_downtime = args.max_flap_downtime
    opt_max_flap_count = args.max_flap_count

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)

    # If no mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use as base LUN")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    error = 0
    for mpath_name in mpath_name_list:
        if not execute_test(mpath_name):
            error += 1

    if error:
        sys.exit(FAIL)
    sys.exit(PASS)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def mask_volume():
    print("INFO: Masking volume.")
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    atomic_run("Masking volume %s to access group %s" % (vol_id, ag_id),
               command=lsm.volume_mask,
               vol=vol_id,
               ag=ag_id,
               errors=errors)
    return errors


if __name__ == "__main__":
    errs = mask_volume()
    exit(parse_ret(errs))



#!/usr/bin/python

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import stqe.host.tc
from libsan.host.linux import install_package
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import vg_create, vg_remove, lv_create, lv_convert, lv_activate
from libsan.host.dmpd import cache_check, cache_dump, cache_restore, cache_repair, get_help, get_version
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run
from six.moves import range

TC = None


def init(args):
    print("INFO: Initializing test case")
    errors = []

    atomic_run("Creating loopdev 1 - 'fast' device",
               name=args["loop1"],
               size=args["loop1_size"],
               command=create_loopdev,
               errors=errors)

    atomic_run("Creating loopdev 2 - 'slow' device",
               name=args["loop2"],
               size=args["loop2_size"],
               command=create_loopdev,
               errors=errors)

    atomic_run("Creating VG",
               vg_name=args["group"],
               pv_name="/dev/" + args["loop1"] +
                       " /dev/" + args["loop2"],
               command=vg_create,
               errors=errors)

    atomic_run("Creating cache metadata volume",
               vg_name=args["group"] + " /dev/" + args["loop1"],
               lv_name=args["meta"],
               options=["-L 12"],
               command=lv_create,
               errors=errors)

    atomic_run("Creating origin volume",
               vg_name=args["group"] + " /dev/" + args["loop2"],
               lv_name=args["origin"],
               options=["-L 2G"],
               command=lv_create,
               errors=errors)

    atomic_run("Creating cache data volume",
               vg_name=args["group"] + " /dev/" + args["loop1"],
               lv_name=args["data"],
               options=["-L 1G"],
               command=lv_create,
               errors=errors)

    atomic_run("Creating cache pool",
               vg_name=args["group"],
               lv_name=args["data"],
               options=["-y --type cache-pool", "--cachemode writeback", "--poolmetadata %s/%s" %
                        (args["group"], args["meta"])],
               command=lv_convert,
               errors=errors)

    atomic_run("Creating cache logical volume",
               vg_name=args["group"],
               lv_name=args["origin"],
               options=["-y", "--type cache", "--cachepool %s/%s" %
                        (args["group"], args["data"])],
               command=lv_convert,
               errors=errors)

    atomic_run("Creating filesystem on cache logical volume",
               cmd="mkfs.ext4 /dev/%s/%s" % (args["group"], args["origin"]),
               command=run,
               errors=errors)

    atomic_run("Splitting cache logical volume",
               vg_name=args["group"],
               lv_name=args["origin"],
               options=["-y", "--splitcache"],
               command=lv_convert,
               errors=errors)

    atomic_run("Creating swap LV",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-L 100"],
               command=lv_create,
               errors=errors)

    atomic_run("Swapping metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--cachepool " + args["group"] + "/" + args["data"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Activating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Initialization passed")
    else:
        TC.tfail("Initialization failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def clean(args):
    print("INFO: Cleaning up")
    errors = []

    atomic_run("Removing VG",
               vg_name=args["group"],
               force=True,
               command=vg_remove,
               errors=errors)

    atomic_run("Deleting loopdev loop1",
               name=args["loop1"],
               command=delete_loopdev,
               errors=errors)

    atomic_run("Deleting loopdev loop2",
               name=args["loop2"],
               command=delete_loopdev,
               errors=errors)

    atomic_run("Deleting metadata file",
               cmd="rm -f /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Deleting repair metadata file",
               cmd="rm -f /var/tmp/metadata_repair",
               command=run,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def install_packages(packages):
    for package in packages:
        install_package(package)


def runtime_test(args):
    print("\n#######################################\n")
    print("INFO: Testing cache tools runtime provided by device_mapper_persistent_data, iteration No. %s" %
           args["iteration"])

    errors = []

    atomic_run("Checking metadata",
               source_lv=args["swap"],
               source_vg=args["group"],
               command=cache_check,
               errors=errors)

    atomic_run("Checking metadata with clear-need-check-flag",
               source_lv=args["swap"],
               source_vg=args["group"],
               clear_needs_check_flag=True,
               command=cache_check,
               errors=errors)

    atomic_run("Checking metadata with super-block-only",
               source_lv=args["swap"],
               source_vg=args["group"],
               super_block_only=True,
               command=cache_check,
               errors=errors)

    atomic_run("Checking metadata with few paramethers",
               source_vg=args["group"],
               source_lv=args["swap"],
               skip_discards=True,
               skip_mappings=True,
               skip_hints=True,
               command=cache_check,
               errors=errors)

    atomic_run("Dumping metadata to standard output",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_dump,
               errors=errors)

    atomic_run("Calculating metadata size for cache of 64 blocks and 128 size",
               cmd="cache_metadata_size --block-size 64 --device-size 128",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for cache of 128 nr blocks",
               cmd="cache_metadata_size --nr-blocks 128 --max-hint-width 4",
               command=run,
               errors=errors)

    atomic_run("Dumping metadata to file",
               source_vg=args["group"],
               source_lv=args["swap"],
               repair=True,
               output="/var/tmp/metadata",
               command=cache_dump,
               errors=errors)

    atomic_run("Checking metadata file",
               False,
               source_file="/var/tmp/metadata",
               command=cache_check,
               errors=errors)

    atomic_run("Restoring metadata with options",
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               quiet=True,
               override_metadata_version=1,
               metadata_version=1,
               command=cache_restore,
               errors=errors)

    atomic_run("Restoring metadata from file",
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=cache_restore,
               errors=errors)

    atomic_run("Repairing metadata to file",
               target_file="/var/tmp/metadata_repair",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_repair,
               errors=errors)

    atomic_run("Repairing metadata from file",
               source_file="/var/tmp/metadata_repair",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=cache_repair,
               errors=errors)

    atomic_run("Simulating TTY for cache_restore",
               cmd="script --return -c 'cache_restore -i /var/tmp/metadata -o /dev/mapper/%s-%s' /dev/null" %
                   (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Checking metadata",
               source_vg=args["group"],
               source_lv=args["swap"],
               quiet=True,
               command=cache_check,
               errors=errors)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing cache tools of device_mapper_persistent_data passed, iteration No. %s" % args["iteration"])
    else:
        TC.tfail("Testing cache tools of device_mapper_persistent_data failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]) +
                 "\nIteration No. %s" % args["iteration"])
        return 1
    return 0


def errors_test(args):
    print("\n#######################################\n")
    print("INFO: Testing cache tools errors provided by device_mapper_persistent_data, iteration No. %s" %
           args["iteration"])

    errors = []

    functions = ["cache_check", "cache_dump", "cache_metadata_size", "cache_repair", "cache_restore"]

    # Sanity to check for missing input
    for func in functions:
        atomic_run("Validating missing input",
                   False,
                   cmd=func,
                   command=run,
                   errors=errors)

    # Sanity to check with wrong input
    for func in functions:
        atomic_run("Validating wrong input",
                   False,
                   cmd=func + " wrong",
                   command=run,
                   errors=errors)

    # Sanity to check with wrong option
    for func in functions:
        atomic_run("Validating wrong option",
                   False,
                   cmd=func + " -wrong",
                   command=run,
                   errors=errors)

    # Sanity to check with wrong -- option
    for func in functions:
        atomic_run("Validating wrong -- option",
                   False,
                   cmd=func + " --wrong",
                   command=run,
                   errors=errors)

    # Sanity to check present functions with -h
    for func in functions:
        atomic_run("Checking help of command",
                   cmd="%s" % func,
                   command=get_help,
                   errors=errors)

    # Sanity to check present functions with -V
    for func in functions:
        atomic_run("Checking version of command",
                   cmd="%s" % func,
                   command=get_version,
                   errors=errors)

    atomic_run("Checking metadata of non-metadata file",
               False,
               cmd="cache_check README",
               command=run,
               errors=errors)

    atomic_run("Checking metadata of non-existent file",
               False,
               cmd="cache_check WRONG",
               command=run,
               errors=errors)

    atomic_run("Checking metadata of non-regular file",
               False,
               cmd="cache_check /dev/mapper/control",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for cache of 64 blocks",
               False,
               cmd="cache_metadata_size --block-size 64",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for cache of 128 size",
               False,
               cmd="cache_metadata_size --device-size 128",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for cache of 64 blocks and 128 size and 128 nr blocks",
               False,
               cmd="cache_metadata_size --block-size 64 --device-size 128 --nr-blocks 128",
               command=run,
               errors=errors)

    atomic_run("Repairing metadata without output",
               False,
               cmd="cache_repair -i /var/tmp/metadata_repair",
               command=run,
               errors=errors)

    atomic_run("Restoring metadata with wrong options",
               False,
               cmd="cache_restore -i /var/tmp/metadata -o /dev/mapper/%s-%s --wrong test" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Restoring metadata with wrong metadata version",
               False,
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               metadata_version=12445,
               command=cache_restore,
               errors=errors)

    atomic_run("Restoring metadata with wrong source",
               False,
               cmd="cache_restore -i /var/tmp/wrong.wrong -o /dev/mapper/%s-%s" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Restoring metadata with bit source",
               False,
               source_file="/var/tmp/metadata_repair",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=cache_restore,
               errors=errors)

    atomic_run("Restoring metadata without output",
               False,
               cmd="cache_restore -i /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Restoring metadata with options",
               pkg=["device-mapper-persistent-data", "0.7.3-2"],
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               omit_clean_shutdown=True,
               command=cache_restore,
               errors=errors)

    atomic_run("Checking metadata",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_check,
               errors=errors)

    # FIXME: Using BZ1255552 for this, find other way
    atomic_run("Corrupting mappings on metadata device",
               False,
               source_file="Makefile",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=cache_restore,
               errors=errors)

    atomic_run("Checking corrupted mappings",
               False,
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_check,
               errors=errors)

    atomic_run("Trying to fail while dumping metadata",
               False,
               source_vg=args['group'],
               source_lv=args['swap'],
               output="/var/tmp/metadata",
               command=cache_dump,
               errors=errors)

    atomic_run("Repairing metadata",
               source_vg=args['group'],
               source_lv=args['swap'],
               target_file="/var/tmp/metadata_repair",
               command=cache_repair,
               errors=errors)

    atomic_run("Corrupting metadata on device",
               cmd="echo 'nothing' >> /dev/mapper/%s-%s" % (args['group'], args['swap']),
               command=run,
               errors=errors)

    atomic_run("Trying to fail while repairing metadata",
               False,
               source_vg=args['group'],
               source_lv=args['swap'],
               target_file="/var/tmp/metadata_repair",
               command=cache_repair,
               errors=errors)

    atomic_run("Trying to fail while dumping metadata",
               False,
               source_vg=args['group'],
               source_lv=args['swap'],
               output="/var/tmp/metadata",
               command=cache_dump,
               errors=errors)

    atomic_run("Checking corrupted metadata",
               False,
               source_vg=args["group"],
               source_lv=args["swap"],
               command=cache_check,
               errors=errors)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing cache tools errors of device_mapper_persistent_data passed, iteration No. %s" %
                 args["iteration"])
    else:
        TC.tfail("Testing cache tools errors of device_mapper_persistent_data failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]) +
                 "\nIteration No. %s" % args["iteration"])
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    args = {"loop1": "loop1",
            "loop1_size": 2048,
            "loop2": "loop2",
            "loop2_size": 4128,
            "group": "vgtest",
            "origin": "origin",
            "data": "cache_data",
            "meta": "cache_meta",
            "pool": "cache_pool",
            "swap": "swapvol"
            }

    # Initialization
    packages = ["device-mapper-persistent-data"]
    install_packages(packages)

    init(args)

    # run it 3 times
    for i in range(1):
        args["iteration"] = i
        runtime_test(args)
        errors_test(args)

    clean(args)

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
from libsan.host.cmdline import run
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def _module_load_unload():
    TestObj.tok("modprobe -r dm_thin_pool")
    print("load & unload dm_thin_pool 100 times")
    TestObj.tok("for i in `seq 100`; do modprobe dm_thin_pool;modprobe -r dm_thin_pool; done")
    # if dm_cache is loaded it also uses dm_persistent_data
    TestObj.tok("modprobe -r dm_cache_cleaner")
    TestObj.tok("modprobe -r dm_cache_smq")
    TestObj.tok("modprobe -r dm_cache")
    TestObj.tok("modprobe -r dm_persistent_data")

    # make sure no thin pool/LV existing and dm-thin-pool & dm-persistent-data are not loaded
    TestObj.tnok("lvs -omodules | grep thin-pool")
    TestObj.tnok("lsmod | egrep -w '^dm_thin_pool'")
    TestObj.tnok("lsmod | egrep -w '^dm_persistent_data'")


def _check_module_value(module, expected_value):
    ret, output = run("lsmod | egrep -w '^%s' | awk '{print $3}'" % module, return_output=True)
    if ret == 0 and output == "%s" % expected_value:
        TestObj.tpass("lsmod | egrep -w '^%s' | awk '{print $3}' == %s" % (module, expected_value))
    else:
        TestObj.tfail("lsmod | egrep -w '^%s' | awk '{print $3}' should return '%s', but returned '%s'" % (
            module, expected_value, output))


def _module_info():
    TestObj.tok("modinfo dm_thin_pool")
    TestObj.tok("modinfo dm_persistent_data")

    TestObj.tok("modinfo -d dm_thin_pool | grep \"^device-mapper thin provisioning target$\"")
    TestObj.tok("modinfo -d dm_persistent_data | grep \"^Immutable metadata library for dm$\"")

    # liner
    TestObj.tok("lvcreate -l1 -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V100m -T %s/pool1 -n lv1" % vg_name)
    _check_module_value("dm_persistent_data", "1")
    _check_module_value("dm_thin_pool", "2")

    TestObj.tok("lvcreate -V100m -T %s/pool1 -n lv2" % vg_name)
    _check_module_value("dm_thin_pool", "3")

    # Reduce active pool
    TestObj.tok("lvcreate -l1 -T %s/pool2" % vg_name)
    TestObj.tok("lvcreate -V100m -T %s/pool2 -n lv21" % vg_name)
    _check_module_value("dm_thin_pool", "5")
    TestObj.tok("lvchange -an %s/lv21" % vg_name)
    _check_module_value("dm_thin_pool", "4")
    TestObj.tok("lvchange -an %s/pool2" % vg_name)
    _check_module_value("dm_thin_pool", "3")

    # stripe
    TestObj.tok("lvcreate -i1 -l1 -T %s/pool3" % vg_name)
    TestObj.tok("lvcreate -V100m -T %s/pool3 -n lv31" % vg_name)
    _check_module_value("dm_thin_pool", "5")
    TestObj.tok("lvcreate -V100m -T %s/pool3 -n lv32" % vg_name)
    _check_module_value("dm_thin_pool", "6")

    # reduce pool with stripes
    TestObj.tok("lvcreate -i2 -l1 -T %s/pool4" % vg_name)
    TestObj.tok("lvcreate -V100m -T %s/pool4 -n lv41" % vg_name)
    _check_module_value("dm_thin_pool", "8")
    TestObj.tok("lvchange -an %s/lv41" % vg_name)
    _check_module_value("dm_thin_pool", "7")
    TestObj.tok("lvchange -an %s/pool4" % vg_name)
    _check_module_value("dm_thin_pool", "6")

    TestObj.tok("lsmod | grep -w dm_thin_pool")
    TestObj.tok("lvs -o +modules %s" % vg_name)

    # should NOT be able to unload the module now
    TestObj.tnok("modprobe -r dm_thin_pool")
    TestObj.tok("lsmod | egrep -w '^dm_thin_pool'")
    TestObj.tok("lsmod | egrep -w '^dm_persistent_data'")

    TestObj.tok("lvremove -ff %s" % vg_name)

    # should be able to unload the module now
    TestObj.tok("modprobe -r dm_thin_pool")
    TestObj.tnok("lsmod | egrep -w '^dm_thin_pool'")
    TestObj.tnok("lsmod | egrep -w '^dm_persistent_data'")


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Provisioning Module test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _module_load_unload()
    _module_info()

    return True


def _clean_up():
    # TestObj.trun("lvremove -ff %s" % vg_name)
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.loopdev import create_loopdev, delete_loopdev
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.stratis import Stratis


def fs_limit():
    errors = []
    stratis = Stratis()
    dev = create_loopdev(size=10240)
    pool_name = "fs_limit"

    atomic_run("Creating pool",
               pool_name=pool_name,
               blockdevs=dev,
               command=stratis.pool_create,
               errors=errors)

    print("INFO: Creating 100 filesystems with 512MiB size.")
    for fs in range(0, 100):
        atomic_run(f"Creating filesystem: fs{fs}",
                   pool_name=pool_name,
                   fs_name=f"fs{fs}",
                   fs_size="512MiB",
                   command=stratis.fs_create,
                   errors=errors
                   )

    atomic_run("Creating fs number 101, this should fail because limit is set to 100 by default",
               expected_out=['ERROR: The pool limit of 100 filesystems has already been reached; increase'
                             ' the filesystem limit on the pool to continue'],
               expected_ret=1,
               pool_name=pool_name,
               fs_name="fs100",
               fs_size="512MiB",
               command=stratis.fs_create,
               errors=errors
               )

    atomic_run("Setting fs limit to 101",
               pool_name=pool_name,
               fs_amount=101,
               command=stratis.pool_set_fs_limit,
               errors=errors
               )

    atomic_run("Creating fs number 101, this should pass because limit has been set to 101",
               pool_name=pool_name,
               fs_name="fs100",
               fs_size="512MiB",
               command=stratis.fs_create,
               errors=errors
               )

    atomic_run("Creating fs number 102, this should fail because limit is set to 101",
               expected_out=['ERROR: The pool limit of 101 filesystems has already been reached; increase'
                             ' the filesystem limit on the pool to continue'],
               expected_ret=1,
               pool_name=pool_name,
               fs_name="fs101",
               fs_size="512MiB",
               command=stratis.fs_create,
               errors=errors
               )

    for fs in range(0, 101):
        atomic_run(f"Destroying filesystem: fs{fs}",
                   pool_name=pool_name,
                   fs_name=f"fs{fs}",
                   command=stratis.fs_destroy,
                   errors=errors
                   )

    atomic_run(f"Destroying pool: {pool_name}",
               pool_name=pool_name,
               command=stratis.pool_destroy,
               errors=errors
               )

    atomic_run("Removing loopdev",
               name=dev,
               command=delete_loopdev,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = fs_limit()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import libsan.host.fio as fio
import libsan.host.scsi as scsi
from stqe.host.persistent_vars import read_var
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.iscsi import discovery_st, node_login, clean_up


def do_io(device):  # fio - running randwrite until the 1G device is full, than verifying written data using CRC32C.
    if not fio.install_fio():
        print('FAIL: Unable to install fio')
        return False

    fio_params = {'bs': '4k', 'direct': 1, 'ioengine': 'libaio', 'iodepth': 16, 'verify': 'crc32c', 'verify_fatal': 1,
                  'do_verify': 1, 'rw': 'randwrite', 'runtime': 120}  # adding runtime cap of 2 minutes

    if not fio.fio_stress(device, verbose=True, **fio_params):
        print('FIO I/O failed')
        return False

    return True


def iscsi_local():
    errors = []
    ip_address = "127.0.0.1"
    target_iqn = read_var("TARGET_IQN")

    atomic_run("Discovering local target",
               target=ip_address,
               disc_db=True,
               command=discovery_st,
               errors=errors
               )

    atomic_run("Trying to login to portal: %s" % ip_address,
               target=target_iqn,
               command=node_login,
               errors=errors
               )

    ret = atomic_run("Getting free scsi disks",
                     filter_only={'vendor': 'LIO-ORG'},
                     command=scsi.get_free_disks,
                     errors=errors
                     )

    try:
        test_dev = '/dev/' + list(ret.keys())[0]
    except IndexError:
        msg = "FAIL: Could not find any free SCSI disk!"
        print(msg)
        errors.append(msg)
        return errors

    atomic_run("Running fio stress on device %s" % test_dev,
               device=test_dev,
               command=do_io,
               errors=errors
               )

    atomic_run("Cleaning up",
               command=clean_up,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = iscsi_local()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_dependants_rm_success():
    errors = []

    src_vol = read_var("LSM_VOL_ID")
    dst_vol = read_var("LSM_VOL_ID_2")
    rep_type = "CLONE"

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Replicating volume %s to volume %s with type %s to create dependency with protocol %s." %
                   (src_vol, dst_vol, rep_type, config['protocol']),
                   command=lsm.volume_replicate_range,
                   src_vol=src_vol,
                   dst_vol=dst_vol,
                   rep_type=rep_type,
                   src_start=0,
                   dst_start=0,
                   count=100,
                   force=True,
                   errors=errors)

        for vol in [src_vol, dst_vol]:
            ret, data = atomic_run("Checking for dependants of volume %s." % vol,
                                   command=lsm.volume_dependants,
                                   vol=vol,
                                   return_output=True,
                                   errors=errors)
            if data != "False" and vol == dst_vol:
                msg = "FAIL: Volume %s should have no dependants." % vol
                print(msg)
                errors.append(msg)

            elif data != "True" and vol == src_vol:
                msg = "FAIL: Volume %s should have dependants." % vol
                print(msg)
                errors.append(msg)

        atomic_run("Removing dependency between replicated volumes %s and %s" % (src_vol, dst_vol),
                   command=lsm.volume_dependants_rm,
                   vol=src_vol,
                   errors=errors)

        for vol in [src_vol, dst_vol]:
            ret, data = atomic_run("Checking for dependants of volume %s." % vol,
                                   command=lsm.volume_dependants,
                                   vol=vol,
                                   return_output=True,
                                   errors=errors)
            if data != "False":
                msg = "FAIL: Volume %s should have no dependants." % vol
                print(msg)
                errors.append(msg)

    return errors


def volume_dependants_rm_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail getting dependants of volume without any paramethers with protocol %s" %
                        config['protocol'], 'command': lsm.volume_dependants_rm},
            {'message': "Trying to fail replicating volume range with WRONG volume with protocol %s" %
                        config['protocol'], 'vol': "WRONG", 'command': lsm.volume_dependants_rm}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_dependants_rm_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_dependants_rm_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from os import environ


def export_fs():
    print("INFO: Creating snapshot of FS.")
    errors = []

    export_path = environ['fmf_fs_export_path']
    rw_host = environ['fmf_rw_host']
    fs_id = read_var("LSM_FS_ID")

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    ret, data = atomic_run("Exporting FS '%s'" % fs_id,
                           command=lsm.fs_export,
                           export_path=export_path,
                           rw_host=rw_host,
                           fs=fs_id,
                           return_output=True,
                           errors=errors)

    for line in data.splitlines():
        if export_path in line:
            export_id = line.split()[0].strip()
            atomic_run("Writing var LSM_EXPORT_ID",
                       command=write_var,
                       var={"LSM_EXPORT_ID": export_id},
                       errors=errors)

    return errors


if __name__ == "__main__":
    errs = export_fs()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env
from stqe.host.fmf_tools import get_env_args, get_func_from_string
from stqe.host.stratis import get_stratis_args


def call_func(args, stratis_object, errors):
    name = read_env("fmf_name")
    if "init_cache/success" in name or "add_cache/success" in name or "add_data/success" in name:
        if "single" in name:
            # Need to take only 1 device
            args['blockdevs'] = args['blockdevs'].pop()
        if "add_cache/success/multiple" in name or "init_cache/success/multiple" in name:
            # using all would trigger error, cache is limited to 32TB
            args['blockdevs'] = args['blockdevs'][:2]
    if "fail/device_instead_of_pool_name" in name or "blockdev/list/fail/device_instead_of_name" in name or \
            "fs/list/fail/device_instead_of_name" in name:
        args['pool_name'] = args['pool_name'].pop()
    if "snapshot/fail/device_instead_of_snapshot_name" in name:
        args['origin_name'] = args['origin_name'].pop()
    if "pool/create/fail/no_pool_name" in name:
        args['blockdevs'] = args['blockdevs'].pop()

    return args


def stratis_cli():
    errors = []
    stratis = Stratis()
    args = get_stratis_args(stratis_object=stratis)
    # some extra args for some setups
    args.update(get_env_args({}))
    args = call_func(args, stratis, errors)
    if args is None:
        return errors

    print("stqe/stratis_cli args is %s" % args)
    command_string = args['command']
    args['command'] = get_func_from_string(stratis, args.pop('command'), local_functions=globals())

    ret = atomic_run(errors=errors,
                     **args)

    check_clean(ret=ret, stratis_object=stratis, args=args, errors=errors, command=command_string)

    return errors


def check_clean(**kwargs):
    if "pool/rename/success" in read_env("fmf_name"):
        _, data = atomic_run("Listing stratis pools.",
                             command=kwargs['stratis_object'].pool_list,
                             return_output=True,
                             errors=kwargs['errors'])
        if kwargs['args']['new'] not in data:
            msg = "FAIL: Did not rename stratis pool %s to %s." % (kwargs['args']['current'], kwargs['args']['new'])
            print(msg)
            kwargs['errors'].append(msg)

    if "fs/rename/success" in read_env("fmf_name"):
        _, data = atomic_run("Listing stratis fs on pool.",
                             command=kwargs['stratis_object'].fs_list,
                             pool_name=kwargs["args"]["pool_name"],
                             return_output=True,
                             errors=kwargs['errors'])
        if kwargs['args']['new_name'] not in data:
            msg = "FAIL: Did not rename stratis pool %s to %s." % (
                kwargs['args']['fs_name'], kwargs['args']['new_name'])
            print(msg)
            kwargs['errors'].append(msg)

    if "key/" in read_env("fmf_name"):
        _, data = atomic_run("Listing stratis pools.",
                             command=kwargs['stratis_object'].pool_list,
                             return_output=True,
                             errors=kwargs['errors'])
        _, data = atomic_run("Listing blockdev .",
                             command=kwargs['stratis_object'].blockdev_list,
                             return_output=True,
                             errors=kwargs['errors'])

        _, data = atomic_run("Listing blockdev .",
                             command=kwargs['stratis_object'].key_list,
                             return_output=True,
                             errors=kwargs['errors'])

    return


if __name__ == "__main__":
    errs = stratis_cli()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.linux import service_start, service_stop
from libsan.host.cmdline import run


def stratis_pid():
    errors = []

    atomic_run("Stopping stratisd service",
               service_name="stratisd",
               command=service_stop,
               errors=errors
               )

    atomic_run("Removing symbolic link /var/run",
               cmd="unlink /var/run",
               command=run,
               errors=errors
               )

    atomic_run("Starting stratisd service",
               service_name="stratisd",
               command=service_start,
               errors=errors
               )

    atomic_run("Restoring symbolic link to /run",
               cmd="ln -s /run /var/run",
               command=run,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = stratis_pid()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from libsan.host.mp import get_disks_of_mpath


def setup_multipath():
    disk = read_var("VDO_DEVICE")
    print("INFO: Setting up dm-multipath on device %s." % disk)
    errors = []

    _, mpaths = atomic_run(message="Getting multipaths device",
                             command=run,
                             return_output=True,
                             cmd="multipath -l -v 1",
                             errors=errors)
    mpaths = [mpath.strip() for mpath in mpaths.splitlines()]
    disk = disk.split("/").pop()

    mpath_disk = disk
    for mpath in mpaths:
        possible_disks = get_disks_of_mpath(mpath)
        if disk in possible_disks:
            mpath_disk = "mapper/%s" % mpath

    atomic_run("Writing var VDO_DEVICE",
               command=write_var,
               var={'VDO_DEVICE': "/dev/%s" % mpath_disk},
               errors=errors)

    atomic_run(message="Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def iscsi_chap_success():
    errors = []

    init = read_var("LSM_INIT_ID_CHAP")
    in_user = read_var("LSM_CHAP_IN_USER")
    in_pass = read_var("LSM_CHAP_IN_PASS")
    out_user = read_var("LSM_CHAP_OUT_USER")
    out_pass = read_var("LSM_CHAP_OUT_PASS")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Setting up CHAP auth without credentials with protocol %s" % config['protocol'],
                   command=lsm.iscsi_chap,
                   init=init,
                   errors=errors)

        atomic_run("Setting up CHAP auth with IN credentials with protocol %s" % config['protocol'],
                   command=lsm.iscsi_chap,
                   init=init,
                   in_user=in_user,
                   in_pass=in_pass,
                   errors=errors)

        atomic_run("Setting up CHAP auth with OUT credentials with protocol %s" % config['protocol'],
                   command=lsm.iscsi_chap,
                   init=init,
                   out_user=out_user,
                   out_pass=out_pass,
                   errors=errors)

        atomic_run("Setting up CHAP auth with all credentials with protocol %s" % config['protocol'],
                   command=lsm.iscsi_chap,
                   init=init,
                   in_user=in_user,
                   in_pass=in_pass,
                   out_user=out_user,
                   out_pass=out_pass,
                   errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = iscsi_chap_success()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.linux import compare_version
from time import localtime, strftime
import os
import json


class Variables:
    # global variables persistent across libraries and instances
    def __init__(self):
        pass

    logging_level = 20
    tests_to_run = "*"
    tier_to_test = 3
    no_run = False
    bz = []  # list of BZ numbers


class Logger:
    def __init__(self):
        pass

    def debug(self, message):
        if Variables.logging_level < 20:
            if message.startswith("\n"):
                print("\nDEBUG: [%s] " % self.get_time() + message[1:])
            else:
                print("DEBUG: [%s] " % self.get_time() + message)

    def info(self, message):
        if message.startswith("\n"):
            print("\nINFO: [%s] " % self.get_time() + message[1:])
        else:
            print("INFO: [%s] " % self.get_time() + message)

    def warn(self, message):
        if message.startswith("\n"):
            print("\nWARN: [%s] " % self.get_time() + message[1:])
        else:
            print("WARN: [%s] " % self.get_time() + message)

    def fail(self, message):
        if message.startswith("\n"):
            print("\nFAIL: [%s] " % self.get_time() + message[1:])
        else:
            print("FAIL: [%s] " % self.get_time() + message)

    @staticmethod
    def get_time():
        return strftime("%Y-%m-%d %H:%M:%S", localtime())


def atomic_run(message, success=True, return_output=False, expected_ret=None, expected_out=None, **kwargs):
    def log_error(error_message, errors_list):
        logger.fail(error_message)
        if isinstance(errors_list, list):
            errors_list.append(error_message)
        else:
            logger.fail("atomic_run got 'errors' value that is not list.")

    def is_expected_out_in_out(out, exp_out):
        out = out.replace("\'", "").replace("\"", "")
        if isinstance(exp_out, list):
            # check all from list are in output
            if all([True if e in out else False for e in exp_out]):
                return True
            else:
                return False
        if str(exp_out) in out:
            return True
        return False

    errors = kwargs.pop("errors")
    command = kwargs.pop("command")

    logger = Logger()

    # used to invert expected output before version of this package, list of ['package', 'version', (optional)'release']
    if 'pkg' in kwargs:
        pkg = [str(x) for x in kwargs.pop('pkg')]
        if len(pkg) == 2:
            pkg.append("")
        comparision = compare_version(*pkg)
        if isinstance(comparision, bool) and not comparision:
            logger.info("Expecting inverse output.")
            if expected_ret is None:
                success = not success
            elif expected_ret != 0:
                expected_ret = 0

    # This is dictionary to switch output to bash logic (0 == success) as in Python True == 1 and not 0
    switcher = {
        True: 0,
        False: 1
    }

    # switch bool to int using switcher to be consistent
    if isinstance(expected_ret, bool):
        try:
            expected_ret = switcher[expected_ret]
        except KeyError:
            pass
    if isinstance(success, bool):
        success = switcher[success]

    # match expected_ret to success in case we do not specify it
    if expected_ret is None:
        expected_ret = success

    params = []
    for a in kwargs:
        params.append(str(a) + " = " + str(kwargs[a]))
    if len(params) != 0:
        params = ", ".join([str(i) for i in params])
        message += " with params %s" % params
    logger.info("\n" + message)

    try:
        ret, output = command(return_output=True, **kwargs)
    except TypeError as e:
        if "takes no keyword arguments" in str(e):
            # this function takes no keyword arguments
            # let's assume there is only 1 kwarg for now
            ret = command(*[kwargs[arg] for arg in kwargs])
            output = ""
        else:
            # unexpected keyword argument return_output, let's try without it
            # and everything else...
            ret = command(**kwargs)
            output = ""
    # '1' gets implemented by python as 'True' and vice versa, switch it back
    if isinstance(ret, bool):
        ret = switcher[ret]

    should = {0: "succeed", 1: "fail"}
    logger.debug("Should %s" % should[success] + " with '%s'. " % expected_ret + "Got '%s'." % ret)
    if (success and ret != expected_ret and type(ret) == type(expected_ret)) or (
                    not success and ret != expected_ret and type(ret) == type(expected_ret)):
        if Variables.logging_level >= 20:
            logger.info("Should %s" % should[success] + " with '%s'. " % expected_ret + "Got '%s'." % ret)
        failed = "failed."
        if success == 1:
            failed = "did not fail."
        error = message + " with params %s %s" % (params, failed)
        log_error(error, errors)

    if expected_out and not is_expected_out_in_out(output, expected_out):
        log_error("Could not find '%s' in returned output, got '%s'." % (expected_out, output), errors)

    if return_output:
        return ret, output
    else:
        return ret


def parse_ret(errors):
    if errors == list():
        return 0
    else:
        # write errors to file for parsing at the end
        try:
            test_name = os.environ['fmf_name']
        except KeyError:
            print("FAIL: Could not get test name though os.environ. Not running error parsing.")
            return 1

        path = '/tmp/test_errors.json'
        if os.path.isfile(path):
            with open(path, 'r') as f:
                dict = json.load(f)
        else:
            dict = {}

        dict[test_name] = errors
        with open(path, 'w') as f:
            json.dump(dict, f)
        return 1



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.mp as mp
import libsan.sanmgmt as sanmgmt
import stqe.host.LogChecker as log
import re
import sys
import subprocess
import argparse


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = "date \"+%Y-%m-%d %H:%M:%S\""
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")

    print("[%s] %s" % (stdout, string))

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def execute_test(mpath_name):
    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    _print("INFO: Going to process the SCSI drivers used by the multipath (%s)" % mpath_name)

    # _print("DEBUG: dump mp_info")
    # print mp_info

    error = False

    if "scsi_drivers" in mp_info:
        for driver in mp_info["scsi_drivers"]:
            _print("INFO: Showing information for %s driver" % driver)
            if run("modinfo %s" % driver) != 0:
                _print("FAIL: Could not show info for %s" % driver)
                error = True
    else:
        _print("FAIL: Could not find out the drivers used by %s" % mpath_name)
        error = True

    if not error:
        _print("INFO: test on %s PASS" % mpath_name)

    if not log.check_all():
        _print("FAIL: detected error on LogChecker")
        return False

    if error:
        _print("FAIL: test case sys/show_driver_info on %s" % mpath_name)
        return False

    _print("PASS: test case sys/show_driver_info on %s" % mpath_name)
    return True


def main():
    """
    """
    PASS = 0
    FAIL = 1

    parser = argparse.ArgumentParser(
        description='add new num')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to query", metavar="mpath")

    args = parser.parse_args()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use as base LUN")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    if not mpath_name_list:
        _print("FAIL: Could not find any multipath device to use")
        sys.exit(FAIL)

    error = 0
    for mpath_name in mpath_name_list:
        if not execute_test(mpath_name):
            error = + 1

    if error:
        sys.exit(FAIL)

    sys.exit(PASS)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from libsan.host.linux import hostname, is_service_running, install_package
from libsan.host.lio import TargetCLI
from libsan.host.iscsi import discovery_st, node_login
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.iscsi import auto_conf
from stqe.host.persistent_vars import write_var
from os import environ
from stqe.host.nfs_lock import setup_nfs_lock_iscsi


def setup_iscsi(test_type="stratis-general"):
    print("INFO: Setting up iSCSI with LUN %s." % test_type)
    errors = []

    install_package("iscsi-initiator-utils")

    if "brq" in hostname():
        auto_conf(test_type=test_type)

    else:
        wwn = f"iqn.1994-05.com.redhat:{test_type}"
        target = TargetCLI(path="/backstores/fileio")
        atomic_run("Creating fileio backstore",
                   command=target.create,
                   name="stratis-test",
                   file_or_dev="/var/tmp/stratis-test.img",
                   size="10G",
                   errors=errors,
                   )

        target.path = "/iscsi"
        atomic_run("Creating iscsi target",
                   wwn=wwn,
                   command=target.create,
                   errors=errors
                   )

        target.path = "/iscsi/" + wwn + "/" + "tpg1" + "/" + "luns"

        atomic_run("Creating lun with fileio backstore",
                   command=target.create,
                   storage_object="/backstores/fileio/stratis-test",
                   errors=errors,
                   )

        target.path = "/iscsi/" + wwn + "/" + "tpg1"

        atomic_run("Setting parameter generate_node_acls to '1'",
                   generate_node_acls=1,
                   group="attribute",
                   command=target.set,
                   errors=errors
                   )

        atomic_run("Setting parameter demo_mode_write_protect to '0'",
                   demo_mode_write_protect=0,
                   group="attribute",
                   command=target.set,
                   errors=errors
                   )

        atomic_run("Discovering local target",
                   target="127.0.0.1",
                   disc_db=True,
                   command=discovery_st,
                   errors=errors
                   )

        atomic_run("Trying to login to portal: 127.0.0.1",
                   portal="127.0.0.1",
                   command=node_login,
                   errors=errors
                   )

    # on RHEL 7 there is no LUN name in /dev/disk/by-path, just ID.
    _, local_disk = atomic_run(message="Getting iSCSI disk",
                               command=run,
                               return_output=True,
                               cmd="ls -la /dev/disk/by-path/ | grep '%s'" % test_type.replace("_", "-"),
                               errors=errors)
    if not local_disk:
        _, local_disk = atomic_run(message="Getting iSCSI disk through iscsiadm",
                                   command=run,
                                   return_output=True,
                                   cmd="iscsiadm -m session -P 3 | grep disk",
                                   errors=errors)
        if local_disk:
            # remove fail from 'ls -la ...'
            errors.pop()
        disk = local_disk.split()[3]
    else:
        disk = local_disk.split("../../").pop()

    if is_service_running("multipathd"):
        ret, multipath = atomic_run(message="Getting mpath device",
                                    command=run,
                                    return_output=True,
                                    cmd="multipath -l /dev/%s -v 1" % disk,
                                    errors=errors)

        atomic_run(message="Flushing mpath device %s" % multipath,
                   command=run,
                   cmd="multipath -f %s" % multipath,
                   errors=errors)

        atomic_run(message="Removing mpath device from disk %s" % disk,
                   command=run,
                   cmd="multipath -W /dev/%s" % disk,
                   errors=errors)

        atomic_run(message="Triggering udev",
                   command=run,
                   cmd="udevadm trigger; udevadm settle",
                   errors=errors)

    if "brq" in hostname():
        ret = atomic_run("NFS locking iscsi disk %s." % disk,
                         command=setup_nfs_lock_iscsi,
                         disk_name=disk,
                         errors=errors)
        if ret:
            print("FAIL: Could not get NFS lock for iscsi LUN.")
            return errors

    atomic_run("Cleaning superblock of device /dev/%s" % disk,
               command=run,
               cmd="dd if=/dev/zero of=/dev/%s bs=1M count=10" % disk,
               errors=errors)

    atomic_run("Writing var STRATIS_DEVICE",
               command=write_var,
               var={'STRATIS_DEVICE': "/dev/%s" % disk},
               errors=errors)

    atomic_run("Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    lun_name = "stratis-general"
    try:
        lun_name = environ['fmf_lun_name']
    except KeyError:
        pass
    errs = setup_iscsi(lun_name)
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.linux as linux
import libsan.host.loopdev as loopdev
from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"
mount_point = "/mnt/thin"


def _convert():
    global TestObj, vg_name, mount_point

    thin_device = "/dev/mapper/%s-thin" % vg_name
    thin_origin_device = "/dev/mapper/%s-thin_origin" % vg_name

    # Convert an LV to be used as thin LV
    TestObj.tok("lvcreate -l25 -n %s/thin" % vg_name)
    linux.mkdir(mount_point)
    fs = linux.get_default_fs()
    if not linux.mkfs(thin_device, fs, force=True):
        TestObj.tfail("Could not create file system (%s)" % fs)
        return False

    if not linux.mount(thin_device, mount_point):
        TestObj.tfail("Could not mount device")
        return False
    TestObj.tok("dd if=/dev/urandom of=%s/5m bs=1M count=5;sync" % mount_point)
    TestObj.tok("md5sum %s/5m > pre_md5" % mount_point)

    TestObj.tok("lvcreate -l50 -T -n %s/pool" % vg_name)
    TestObj.tlog("test case:1")
    # https://bugzilla.redhat.com/show_bug.cgi?id=997704#c5
    TestObj.tok("lvconvert --thinpool %s/pool --thin %s/thin --originname thin_origin -y" % (vg_name, vg_name))
    linux.sync()
    # Make sure there was no data corruption
    TestObj.tlog("1.1 checking if the md5 checksum is not changed")
    TestObj.tok("md5sum %s/5m > post_md5" % mount_point)
    TestObj.tok("diff pre_md5 post_md5")

    TestObj.tlog("1.2 checking if the thin LV is converted")
    check_lv_expected_value(TestObj, "thin", vg_name,
                            {"lv_size": "100.00m", "pool_lv": "pool",
                             "lv_attr": "Vwi-aotz--", "origin": "thin_origin"})

    TestObj.tlog("1.3 checking if a readonly lv is created for the pre-data")
    check_lv_expected_value(TestObj, "thin_origin", vg_name, {"lv_attr": "ori-------"})

    TestObj.tlog("1.4 checking if the new data will be stored in the pool")
    lv_info = lvm.lv_info("thin", vg_name, "data_percent")
    if not lv_info:
        TestObj.tfail("Could not query data_percent from thin LV")
        return False
    pre_thin_dp = float(lv_info["data_percent"])
    lv_info = lvm.lv_info("pool", vg_name, "data_percent")
    if not lv_info:
        TestObj.tfail("Could not query data_percent from thin LV")
        return False
    pre_pool_dp = float(lv_info["data_percent"])

    TestObj.tok("dd if=/dev/urandom of=%s/10m bs=1M count=10;sync" % mount_point)
    lv_info = lvm.lv_info("thin", vg_name, "data_percent")
    if not lv_info:
        TestObj.tfail("Could not query data_percent from thin LV")
        return False
    post_thin_dp = float(lv_info["data_percent"])
    lv_info = lvm.lv_info("pool", vg_name, "data_percent")
    if not lv_info:
        TestObj.tfail("Could not query data_percent from thin LV")
        return False
    post_pool_dp = float(lv_info["data_percent"])
    if post_thin_dp > pre_thin_dp and post_pool_dp > pre_pool_dp:
        TestObj.tpass("Data percentage increased correctly")
    else:
        TestObj.tfail("Data percentage did not increase")

    TestObj.tlog("1.5 checking deleting the pre-data, the origin will not impact")
    TestObj.tok("rm -rf %s/5m" % mount_point)
    if not linux.umount(mount_point):
        TestObj.tfail("Could not umount device")
        return False

    TestObj.tok("lvremove -ff %s/thin" % vg_name)
    TestObj.tok("lvchange -ay %s/thin_origin" % vg_name)

    if fs == "xfs":
        # XFS needs  'writable' device to  be able to work with journal
        TestObj.tok("lvchange -prw %s/thin_origin" % vg_name)

    if not linux.mount(thin_origin_device, mount_point):
        TestObj.tfail("Could not mount thin origin device")
        return False
    TestObj.tok("md5sum %s/5m > origin_md5" % mount_point)
    TestObj.tok("diff pre_md5 origin_md5")


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Pool Convert test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _convert()

    return True


def _clean_up():
    linux.umount(mount_point)

    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        print(e)
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_snap_create_success():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Creating snapshot %s on FS %s with protocol %s." %
                               (snap_name, fs_id, config['protocol']),
                               command=lsm.fs_snap_create,
                               name=snap_name,
                               fs=fs_id,
                               return_output=True,
                               errors=errors)
        snap_id = [line.split()[0].strip() for line in data.splitlines() if snap_name in line][0]
        atomic_run("Removing snapshot %s from FS %s with protocol %s." % (snap_id, fs_id, config['protocol']),
                   command=lsm.fs_snap_delete,
                   snap=snap_id,
                   fs=fs_id,
                   force=True,
                   errors=errors)
    return errors


def fs_snap_create_fail():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail creating snapshot on FS without any paramethers with protocol %s"
                        % config['protocol'], 'command': lsm.fs_snap_create},
            {'message': "Trying to fail creating snapshot on FS without 'fs' with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_create, 'name': snap_name},
            {'message': "Trying to fail creating snapshot on FS without 'name' with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_create, 'fs': fs_id},
            {'message': "Trying to fail creating snapshot on FS by using fs-name instead of fs-ID with protocol %s"
                        % config['protocol'], 'command': lsm.fs_snap_create, 'fs': fs_name, 'name': snap_name},
            {'message': "Trying to fail creating snapshot on NONEXISTENT FS with protocol %s" % config['protocol'],
             'command': lsm.fs_snap_create, 'fs': "NONEXISTENT", 'name': snap_name},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def fs_snap_create_fail_name_conflict():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Creating snapshot %s on FS %s with protocol %s." %
                               (snap_name, fs_id, config['protocol']),
                               command=lsm.fs_snap_create,
                               name=snap_name,
                               fs=fs_id,
                               return_output=True,
                               errors=errors)
        snap_id = [line.split()[0].strip() for line in data.splitlines() if snap_name in line][0]

        atomic_run("Creating snapshot %s on FS %s again to hit name conflict with protocol %s." %
                   (snap_name, fs_id, config['protocol']),
                   command=lsm.fs_snap_create,
                   name=snap_name,
                   fs=fs_id,
                   expected_ret=4,
                   errors=errors)

        atomic_run("Removing snapshot %s from FS %s with protocol %s." % (snap_id, fs_id, config['protocol']),
                   command=lsm.fs_snap_delete,
                   snap=snap_id,
                   fs=fs_id,
                   force=True,
                   errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_snap_create_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_snap_create_fail()
        errs += fs_snap_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def access_group_volumes_success():
    errors = []

    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Listing volumes with access from AG %s with protocol %s." % (ag_id, config['protocol']),
                   command=lsm.access_group_volumes,
                   ag=ag_id,
                   errors=errors)

    return errors


def access_group_volumes_fail():
    errors = []

    ag_name = read_var("LSM_AG_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail listing volumes with access from AG without any paramethers "
                        "with protocol %s" % config['protocol'], 'command': lsm.access_group_volumes},
            {'message': "Trying to fail listing volumes with access from AG with AG NONEXISTENT with protocol %s" % config[
                'protocol'], 'command': lsm.access_group_volumes, 'ag': "NONEXISTENT"},
            {'message': "Trying to fail listing volumes with access from with AG name instead of ID with protocol %s" %
                        config['protocol'], 'command': lsm.access_group_volumes, 'ag': ag_name}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = access_group_volumes_success()
    if int(environ['fmf_tier']) == 2:
        errs = access_group_volumes_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import parse_ret
from stqe.host.persistent_vars import read_var, clean_var, write_var


def setup_manual():
    # this just checks if the device is created.
    data = read_var('STRATIS_DEVICE')

    # remove newline
    if data.endswith('\n'):
        data = data.rstrip('\n')
        clean_var('STRATIS_DEVICE')
        write_var({'STRATIS_DEVICE': data})

    print("INFO: Will run tests on device: '%s'" % data)
    return []


if __name__ == "__main__":
    errs = setup_manual()
    exit(parse_ret(errs))
